@@ -1,10 +1,16 @@
+"""This file provides the fExpr class to represent f-domain (Fourier
+domain) expressions.
+
+Copyright 2014--2019 Michael Hayes, UCECE
+
+"""
+
 from __future__ import division
 from .fourier import inverse_fourier_transform
-from .sfwexpr import sfwExpr
+from .expr import Expr
 from .sym import fsym, ssym, tsym
-#import .texpr as texpr
 
-class fExpr(sfwExpr):
+class fExpr(Expr):
 
     """Fourier domain expression or symbol."""
 
@@ -45,9 +51,24 @@ class fExpr(sfwExpr):
         return self.time(**assumptions).laplace()
     
     def plot(self, fvector=None, **kwargs):
-        """Plot frequency response at values specified by fvector.
+        """Plot frequency response at values specified by fvector.  If fvector
+        is a tuple, this sets the frequency limits.
 
-        There are many plotting options, see matplotlib.pyplot.plot.
+        kwargs include:
+        axes - the plot axes to use otherwise a new figure is created
+        xlabel - the x-axis label
+        ylabel - the y-axis label
+        ylabel2 - the second y-axis label if needed, say for mag and phase
+        xscale - the x-axis scaling, say for plotting as ms
+        yscale - the y-axis scaling, say for plotting mV
+        plot_type -  'dB_phase', 'mag-phase', 'real-imag', 'mag', 'phase',
+        'real', or 'imag'
+        in addition to those supported by the matplotlib plot command.
+        
+        The plot axes are returned.
+
+        There are many plotting options, see lcapy.plot and
+        matplotlib.pyplot.plot.
 
         For example:
             V.plot(fvector, log_frequency=True)
@@ -56,6 +77,7 @@ class fExpr(sfwExpr):
 
         By default complex data is plotted as separate plots of magnitude (dB)
         and phase.
+
         """
 
         from .plot import plot_frequency
@@ -126,5 +148,13 @@ class If(fExpr):
         super(If, self).__init__(val, **assumptions)
         self._fourier_conjugate_class = It
 
+
+def fexpr(arg):
+    """Create fExpr object.  If `arg` is fsym return f"""
+
+    if arg is fsym:
+        return f
+    return fExpr(arg)
+        
 from .texpr import Ht, It, Vt, Yt, Zt, tExpr
 f = fExpr('f')
