@@ -26,6 +26,8 @@ from .sym import sympify, simplify
 from .utils import factor_const, scale_shift
 import sympy as sym
 
+__all__ = ('LT', 'ILT')
+
 laplace_cache = {}
 inverse_laplace_cache = {}
 
@@ -129,6 +131,20 @@ def laplace_integral(expr, t, s):
     
     return F1 * F2
 
+def laplace_derivative_undef(expr, t, s):
+    
+    if not isinstance(expr, sym.Derivative):
+        raise ValueError('Cannot compute Laplace transform of %s' % expr)
+    
+    if (not isinstance(expr.args[0], sym.function.AppliedUndef) and
+        expr.args[1][0] != t):
+        raise ValueError('Cannot compute Laplace transform of %s' % expr)
+
+    ssym = sympify(str(s))    
+    name = expr.args[0].func.__name__    
+    func1 = name[0].upper() + name[1:] + '(%s)' % str(ssym)    
+    return sympify(func1).subs(ssym, s) * s ** expr.args[1][1]
+
 def laplace_term(expr, t, s):
 
     const, expr = factor_const(expr, t)
@@ -138,9 +154,12 @@ def laplace_term(expr, t, s):
 
     if expr.has(sym.Integral):
         return laplace_integral(expr, t, s) * const
-    
+
     if expr.has(sym.function.AppliedUndef):
 
+        if expr.has(sym.Derivative):
+            return laplace_derivative_undef(expr, t, s) * const    
+
         rest = sym.S.One
         expr = expr.cancel()
         for factor in expr.as_ordered_factors():
@@ -168,10 +187,12 @@ def laplace_term(expr, t, s):
 def laplace_transform(expr, t, s):
     """Compute unilateral Laplace transform of expr with lower limit 0-.
 
-    Undefined functions such as v(t) are converted to V(s)
-
-    """
+    Undefined functions such as v(t) are converted to V(s)."""
 
+    if expr.is_Equality:
+        return sym.Eq(laplace_transform(expr.args[0], t, s),
+                      laplace_transform(expr.args[1], t, s))
+    
     key = (expr, t, s)
     if key in laplace_cache:
         return laplace_cache[key]
@@ -190,8 +211,8 @@ def laplace_transform(expr, t, s):
     else:
         expr = sympify(expr)
 
-    # Unilateral LT ignores expr for t < 0 so
-    # but barfs on a Piecewise so handle case here.
+    # SymPy laplace barfs on Piecewise but unilateral LT ignores expr
+    # for t < 0 so remove Piecewise.
     expr = expr.replace(var, t)        
     if expr.is_Piecewise and expr.args[0].args[1].has(t >= 0):
         expr = expr.args[0].args[0]
@@ -211,12 +232,67 @@ def laplace_transform(expr, t, s):
     return result
 
 
-def inverse_laplace_ratfun(expr, s, t):
+def inverse_laplace_damped_sin(expr, s, t, **assumptions):
 
-    N, D, delay = Ratfun(expr, s).as_ratfun_delay()
-    # The delay should be zero
+    ncoeffs, dcoeffs = expr.coeffs()
+    K = ncoeffs[0] / dcoeffs[0]
 
-    Q, M = sym.div(N, D, s)
+    ncoeffs = [(c / ncoeffs[0]) for c in ncoeffs]
+    dcoeffs = [(c / dcoeffs[0]) for c in dcoeffs]        
+
+    if len(ncoeffs) > 3 or len(dcoeffs) > 3:
+        raise ValueError('Not a second-order response')
+    
+    omega0 = sym.sqrt(dcoeffs[2])
+    zeta = dcoeffs[1] / (2 * omega0)
+
+    if zeta.is_constant() and zeta > 1:
+        print('Warning: expression is overdamped')
+
+    sigma1 = (zeta * omega0).simplify()
+    omega1 = (omega0 * sym.sqrt(1 - zeta**2)).simplify()
+    K = (K / omega1).simplify()
+
+    E = sym.exp(-sigma1 * t)
+    S = sym.sin(omega1 * t)
+
+    h = K * E * S
+
+    # If overdamped
+    #h = K * sym.exp(-sigma1 * t) * sym.sinh(omega0 * mu * t)
+        
+    if len(ncoeffs) == 1:
+        return sym.S.Zero, h
+
+    C = sym.cos(omega1 * t)
+    kCd = omega1
+    kSd = -sigma1
+    hd = K * E * (kCd * C + kSd * S)
+    
+    if len(ncoeffs) == 2:
+        return sym.S.Zero, K * E * (kCd * C + (ncoeffs[1] + kSd) * S)
+
+    kCdd = -2 * omega1 * sigma1
+    kSdd = sigma1**2 - omega1**2
+
+    G = K * E * ((kCdd + ncoeffs[1] * kCd) * C + (kSdd + ncoeffs[1] * kSd + ncoeffs[2]) * S)
+    
+    return K * kCd * sym.DiracDelta(t), G
+
+
+def inverse_laplace_ratfun(expr, s, t, **assumptions):
+
+    sexpr = Ratfun(expr, s)
+
+    damping = assumptions.get('damping', None)
+
+    if assumptions.get('damped_sin', False):
+        if sexpr.degree == 2:
+            return inverse_laplace_damped_sin(sexpr, s, t, **assumptions)
+        if False and sexpr.degree == 3 and Ratfun(expr * s).degree == 2:
+            return inverse_laplace_damped_sin3(sexpr, s, t, **assumptions)
+
+    Q, M, D, delay, undef = sexpr.as_QMD()
 
     result1 = sym.S.Zero
 
@@ -232,31 +308,34 @@ def inverse_laplace_ratfun(expr, s, t):
             return factor
 
     sexpr = Ratfun(expr, s)
-    P = sexpr.poles()
+    poles = sexpr.poles(damping=damping)
+    polesdict = {}
+    for pole in poles:
+        polesdict[pole.expr] = pole.n
+    
     result2 = sym.S.Zero
 
-    P2 = P.copy()
+    for pole in poles:
 
-    for p in P2:
+        p = pole.expr
 
         # Number of occurrences of the pole.
-        N = P2[p]
+        o = polesdict[p]        
 
-        if N == 0:
+        if o == 0:
             continue
 
-        f = s - p
-
-        if N == 1:
-            r = sexpr.residue(p, P)
-
-            pc = p.conjugate()
-            if pc != p and pc in P:
+        if o == 1:
+            pc = pole.conjugate
+            r = sexpr.residue(p, poles)
+            
+            if pc != p and pc in polesdict:
                 # Remove conjugate from poles and process pole with its
                 # conjugate.  Unfortunately, for symbolic expressions
                 # we cannot tell if a quadratic has two real poles,
-                # a repeat real pole, or a complex conjugate pair of poles.
-                P2[pc] = 0
+                # a repeated real pole, or a complex conjugate pair of poles.
+
+                polesdict[pc] -= 1
                 
                 p_re = sym.re(p)
                 p_im = sym.im(p)
@@ -270,9 +349,10 @@ def inverse_laplace_ratfun(expr, s, t):
             continue
 
         # Handle repeated poles.
-        expr2 = expr * f ** N
-        for n in range(1, N + 1):
-            m = N - n
+        expr2 = expr * (s - p) ** o
+        expr2 = expr2.simplify()        
+        for n in range(1, o + 1):
+            m = o - n
             r = sym.limit(
                 sym.diff(expr2, s, m), s, p) / sym.factorial(m)
             result2 += r * sym.exp(p * t) * t**(n - 1)
@@ -361,6 +441,27 @@ def inverse_laplace_product(expr, s, t, **assumptions):
     return result * const
 
 
+def inverse_laplace_power(expr, s, t, **assumptions):
+
+    # Handle expressions with a power of s.
+    if not (expr.is_Pow and expr.args[0] == s):
+        raise ValueError('Expression %s is not a power of s' % expr)
+    exponent = expr.args[1]
+
+    # Have many possible forms; the common ones are:
+    # s**a, s**-a, s**(1+a), s**(1-a), s**-(1+a), s**(a-1)
+    # Cannot tell if 1-a is positive.
+
+    if exponent.is_positive:
+        # Unfortunately, SymPy does not seem to support fractional
+        # derivatives...
+        return sym.Derivative(sym.DiracDelta(t), t, exponent, evaluate=False)
+
+    if exponent.is_negative:
+        return sym.Pow(t, -exponent - 1) / sym.Gamma(-exponent)
+
+    raise ValueError('Cannot determine sign of exponent for %s' % expr)
+
 def delay_factor(expr, var):
 
     delay = sym.S.Zero    
@@ -410,7 +511,7 @@ def inverse_laplace_term1(expr, s, t, **assumptions):
 
     try:
         # This is the common case.
-        result1, result2 = inverse_laplace_ratfun(expr, s, t)
+        result1, result2 = inverse_laplace_ratfun(expr, s, t, **assumptions)
         return const * result1, const * result2
     except:
         pass
@@ -420,13 +521,16 @@ def inverse_laplace_term1(expr, s, t, **assumptions):
     except:
         pass
 
+    if expr.is_Pow and expr.args[0] == s:
+        return sym.S.Zero, const * inverse_laplace_power(expr, s, t)
+    
     # As last resort see if can convert to convolutions...
     return sym.S.Zero, const * inverse_laplace_product(expr, s, t)
     
     
 def inverse_laplace_term(expr, s, t, **assumptions):
 
-    expr, delay = delay_factor(simplify(expr), s)
+    expr, delay = delay_factor(expr, s)
 
     result1, result2 = inverse_laplace_term1(expr, s, t, **assumptions)
 
@@ -445,7 +549,6 @@ def inverse_laplace_term(expr, s, t, **assumptions):
 
 def inverse_laplace_by_terms(expr, s, t, **assumptions):
 
-    expr = sym.expand(expr)
     terms = expr.as_ordered_terms()
 
     result1 = sym.S.Zero
@@ -470,10 +573,18 @@ def inverse_laplace_transform(expr, s, t, **assumptions):
     ac -- x(t) = A cos(a * t) + B * sin(b * t)
     """
 
+    if expr.is_Equality:
+        return sym.Eq(inverse_laplace_transform(expr.args[0], s, t,
+                                                **assumptions),
+                      inverse_laplace_transform(expr.args[1], s, t,
+                                                **assumptions))    
+
     # TODO, simplify
     key = (expr, s, t, assumptions.get('dc', False),
            assumptions.get('ac', False),
-           assumptions.get('causal', False))
+           assumptions.get('causal', False),
+           assumptions.get('damping', None),           
+           assumptions.get('damped_sin', None))
     
     if key in inverse_laplace_cache:
         return inverse_laplace_cache[key]
@@ -490,11 +601,16 @@ def inverse_laplace_transform(expr, s, t, **assumptions):
                              ' Perhaps have capacitors in series?')
         return result
 
-    try:
-        result1, result2 = inverse_laplace_term(expr, s, t, **assumptions)
-    except:
+    if expr.is_Add:
         result1, result2 = inverse_laplace_by_terms(expr, s, t, **assumptions)
-        
+    else:
+        try:
+            result1, result2 = inverse_laplace_term(expr, s, t, **assumptions)
+        except:
+            expr = sym.expand(expr)
+            result1, result2 = inverse_laplace_by_terms(expr, s, t,
+                                                        **assumptions)
+
     # result1 is known to be causal, result2 is unsure
     result = result1 + result2    
 
@@ -510,4 +626,27 @@ def inverse_laplace_transform(expr, s, t, **assumptions):
     inverse_laplace_cache[key] = result
     return result
 
+
+def LT(expr, t, s, **assumptions):
+    """Compute unilateral Laplace transform of expr with lower limit 0-.
+
+    Undefined functions such as v(t) are converted to V(s)."""
+    
+    return laplace_transform(expr, t, s, **assumptions)
+
+
+def ILT(expr, s, t, **assumptions):
+    """Calculate inverse Laplace transform of X(s) and return x(t).
+
+    The unilateral Laplace transform cannot determine x(t) for t < 0
+    unless given additional information in the way of assumptions.
+
+    The assumptions are:
+    dc -- x(t) = constant so X(s) must have the form constant / s
+    causal -- x(t) = 0 for t < 0.
+    ac -- x(t) = A cos(a * t) + B * sin(b * t)
+    """
+    
+    return inverse_laplace_transform(expr, s, t, **assumptions)
+
 from .expr import Expr
