@@ -9,13 +9,16 @@ Copyright 2015--2019 Michael Hayes, UCECE
 from __future__ import print_function
 from .cexpr import cExpr
 from .omegaexpr import omegaExpr
-from .symbols import j, omega, jomega
+from .symbols import j, omega, jomega, s, t
 from .functions import sqrt
 from .sym import capitalize_name, omegasym
 from .grammar import delimiters
+from .immitance import ImmitanceMixin
+from .current import Current
 import lcapy
 import inspect
 import sys
+import sympy as sym
 
 module = sys.modules[__name__]
 
@@ -31,21 +34,7 @@ def arg_format(value):
     return string
 
 
-def _YZtype_select(expr, kind):
-    """Return appropriate admittance/impedance value for analysis kind."""
-
-    if kind in ('s', 'ivp', 'super', 'laplace'):
-        return expr
-    elif kind in ('dc', 'time'):
-        return cExpr(expr.subs(0))
-    elif isinstance(kind, str) and kind[0] == 'n':
-        return expr(jomega)
-    elif kind in (omegasym, omega, 'ac'):
-        return expr(jomega)
-    return omegaExpr(expr.subs(j * kind))
-    
-
-class Cpt(object):
+class Cpt(ImmitanceMixin):
 
     dependent_source = False
     independent_source = False    
@@ -53,13 +42,16 @@ class Cpt(object):
     need_branch_current = False
     need_extra_branch_current = False    
     need_control_current = False
+    directive = False
+    flip_branch_current = False    
 
-    def __init__(self, cct, namespace, name, cpt_type, cpt_id, string,
+    def __init__(self, cct, namespace, defname, name, cpt_type, cpt_id, string,
                  opts_string, nodes, keyword, *args):
 
         self.cct = cct
         self.type = cpt_type
         self.id = cpt_id
+        self.defname = defname
         self.name = name
         self.relname = name
         self.namespace = ''
@@ -76,7 +68,8 @@ class Cpt(object):
                     node = node[len(self.namespace):]
                 self.relnodes.append(node)
 
-        self.net = string.split(';')[0]
+        self.string = string
+        #self.net = string.split(';')[0]
         # This is the initial opts_string from which the opts attribute
         # is derived.
         self.opts_string = opts_string
@@ -87,10 +80,13 @@ class Cpt(object):
         self.opts = {}
 
         # No defined cpt
-        if self.type in ('W', 'O', 'P', 'K'):
+        if self.type in ('XX', ):
+            self.cpt = lcapy.oneport.Dummy()
             return
 
-        if args is () or (self.type in ('F', 'H') and len(args) == 1):
+        if ((args is () and not self.type in ('W', 'O', 'P'))
+            or (self.type in ('F', 'H', 'CCCS', 'CCVS') and len(args) == 1)
+            or (self.type == 'K' and len(args) == 2)):
             # Default value is the component name
             value = self.type + self.id
 
@@ -100,11 +96,19 @@ class Cpt(object):
             args += (value, )
             self.args = args
 
+        classname = self.classname
+        # Handle aliases.
         try:
-            newclass = getattr(lcapy.oneport, self.classname)
+            classname = {'E' : 'VCVS', 'F' : 'CCCS',
+                         'G' : 'VCCS',  'H' : 'CCVS'}[classname]
+        except:
+            pass
+                
+        try:
+            newclass = getattr(lcapy.oneport, classname)
         except:
             try:
-                newclass = getattr(lcapy.twoport, self.classname)
+                newclass = getattr(lcapy.twoport, classname)
             except:
                 return
                 
@@ -114,10 +118,7 @@ class Cpt(object):
         return self.__str__()
 
     def __str__(self):
-
-        if self.opts == {}:
-            return self.net
-        return self.net + '; ' + str(self.opts)
+        return self.string
 
     def stamp(self, cct):
         raise NotImplementedError('stamp method not implemented for %s' % self)
@@ -127,21 +128,16 @@ class Cpt(object):
         
         return str(self)
     
-    def kill_initial(self):
-        """Kill implicit sources due to initial conditions."""
+    def kill(self):
+        """Kill sources."""
 
         return self.copy()
 
-    def kill(self):
-        """Kill component."""
-
-        raise ValueError('component not a source: %s' % self)
-
     def netmake(self, node_map=None, zero=False):
         """Create a new net description.  If node_map is not None,
         rename the nodes.  If zero is True, set args to zero."""
 
-        string = self.name
+        string = self.defname
         field = 0
         
         for node in self.relnodes:
@@ -186,7 +182,7 @@ class Cpt(object):
         return self.copy()
 
     def ss_model(self):
-        """Return preliminary state-space model of component."""
+        """Return state-space model of component."""
 
         return self.copy()            
 
@@ -195,6 +191,11 @@ class Cpt(object):
 
         return self.copy()        
 
+    @property
+    def is_source(self):
+        """Return True if component is a source (dependent or independent)"""
+        return self.dependent_source or self.independent_source
+
     @property
     def is_causal(self):
         """Return True if causal component or if source produces
@@ -238,7 +239,18 @@ class Cpt(object):
         elif self.cpt.current_source:
             return self.cpt.Isc.has_s
         else:
-            raise ValueError('%s is not a source' % self)        
+            raise ValueError('%s is not a source' % self)
+
+    @property
+    def has_transient(self):
+        """Return True if source has a transient component."""
+
+        if self.cpt.voltage_source:
+            return self.cpt.Voc.has_transient
+        elif self.cpt.current_source:
+            return self.cpt.Isc.has_transient
+        else:
+            raise ValueError('%s is not a source' % self)                
         
     @property
     def is_noisy(self):
@@ -265,13 +277,17 @@ class Cpt(object):
 
     @property
     def I(self):
-        """Current through component."""
+        """Current through component.  The current is defined to be into the
+        positive node for passive devices and out of the positive node
+        for sources."""
 
         return self.cct.get_I(self.name)
 
     @property
     def i(self):
-        """Time-domain current through component."""
+        """Time-domain current through component.  The current is
+        defined to be into the positive node for passive devices and
+        out of the positive node for sources."""
 
         return self.cct.get_i(self.name)
 
@@ -289,55 +305,118 @@ class Cpt(object):
 
     @property
     def Isc(self):
-        """Short-circuit current."""
+        """Short-circuit current for component in isolation, i.e, current in
+        wire connected across component."""
 
         return self.cpt.Isc.select(self.cct.kind)
 
     @property
     def isc(self):
-        """Short-circuit time-domain current for circuit kind."""
+        """Short-circuit time-domain current for component in isolation, i.e,
+        current in wire connected across component."""
 
         return self.Isc.time()
 
     @property
     def Voc(self):
-        """Open-circuit voltage."""
+        """Open-circuit voltage for component in isolation."""
 
         return self.cpt.Voc.select(self.cct.kind)
 
     @property
     def voc(self):
-        """Open-circuit time-domain voltage for circuit kind."""
+        """Open-circuit time-domain voltage for component in isolation."""
 
         return self.Voc.time()    
-
-    @property
-    def Ys(self):
-        """Admittance (s-domain)"""
-        return self.cpt.Y
-
-    @property
-    def Zs(self):
-        """Impedance (s-domain)"""
-        return self.cpt.Z
     
     @property
-    def Y(self):
-        """Admittance"""
+    def V0(self):
+        """Initial voltage (for capacitors only)."""
 
-        return _YZtype_select(self.Ys, self.cct.kind)
+        return 0
 
     @property
-    def Z(self):
-        """Impedance"""
+    def I0(self):
+        """Initial current (for inductors only)."""
 
-        return _YZtype_select(self.Zs, self.cct.kind)        
+        return 0    
+    
+    @property
+    def admittance(self):
+        """Self admittance of component.  For the driving-point
+        admittance measured across the component use .dpY or .oneport().Y"""
 
+        return self.cpt.admittance.new(self.cct.kind)        
+
+    @property
+    def impedance(self):
+        """Self impedance of component.  For the driving-point impedance
+        measured across the component use .dpZ or .oneport().Z"""        
+
+        return self.cpt.impedance.new(self.cct.kind)
+
+    @property
+    def dpIsc(self):
+        """Driving-point short-circuit current, i.e, current in wire
+        connected across component connected in-circuit.
+
+        """
+        return self.oneport().Isc
+
+    @property
+    def dpisc(self):
+        """Driving-point short-circuit time-domain current i.e, current in
+        wire connected across component in-circuit."""
+
+        return self.dpIsc.time()
+
+    @property
+    def dpVoc(self):
+        """Driving-point open-circuit voltage across component in-circuit."""
+
+        return self.oneport().V
+
+    @property
+    def dpvoc(self):
+        """Driving-point open-circuit time-domain voltage across component in
+        circuit."""
+
+        return self.dpVoc.time()    
+
+    @property
+    def dpYs(self):
+        """Driving-point generalized admittance (s-domain) measured across
+        component in-circuit.  For the admittance of the component in isolation
+        use .Ys"""
+
+        return self.oneport().Ys
+
+    @property
+    def dpZs(self):
+        """Driving-point generalized impedance (s-domain) measured across
+        component in-circuit.  For the impedance of the component in isolation
+        use .Zs"""        
+
+        return self.oneport().Zs
+    
+    @property
+    def dpY(self):
+        """Driving-point admittance measured across component in-circuit.  For
+        the admittance of the component in isolation use .Y"""        
+
+        return self.oneport().Y
+
+    @property
+    def dpZ(self):
+        """Driving-point impedance measured across component in-circuit.  For
+        the impedance of the component in isolation use .Z"""
+
+        return self.oneport().Z
 
     @property
     def node_indexes(self):
 
-        return (self.cct._node_index(n) for n in self.nodes)
+        return [self.cct._node_index(n) for n in self.nodes]
 
     @property
     def branch_index(self):
@@ -348,6 +427,21 @@ class Cpt(object):
 
         return '_' + self.cct._make_anon('node')
 
+    def oneport(self):
+        """Create oneport object."""
+        
+        return self.cct.oneport(self.nodes[1], self.nodes[0])
+
+    def thevenin(self):
+        """Create Thevenin oneport object."""
+        
+        return self.cct.thevenin(self.nodes[1], self.nodes[0])
+
+    def norton(self):
+        """Create Norton oneport object."""
+        
+        return self.cct.norton(self.nodes[1], self.nodes[0])    
+    
 
 class Invalid(Cpt):
     
@@ -389,6 +483,13 @@ class Dummy(Cpt):
     hasic = None
     noisy = False
 
+
+class XX(Dummy):
+    directive = True
+    
+    def stamp(self, cct):
+        pass
+
     
 class IndependentSource(Cpt):
 
@@ -408,17 +509,9 @@ class IndependentSource(Cpt):
 class DependentSource(Dummy):
 
     dependent_source = True        
-    
-    def zero(self):
-        """Zero value of the source.  For a voltage source this makes it a
-        short-circuit; for a current source this makes it
-        open-circuit.  This effectively kills the source but keeps it
-        as a source in the netlist.  This is required for dummy
-        voltage sources that are required to specify the controlling
-        current for CCVS and CCCS components.
 
-        """
-        return self.netmake(zero=True)    
+    def zero(self):
+        return self.copy()
 
     
 class RLC(Cpt):
@@ -428,7 +521,7 @@ class RLC(Cpt):
         if self.Voc == 0:        
             return '%sZ%s %s %s %s; %s' % (self.namespace, self.relname, 
                                            self.relnodes[0], self.relnodes[1],
-                                           arg_format(self.Zs(var)), 
+                                           arg_format(self.Z(var)), 
                                            self.opts)
 
         dummy_node = self.dummy_node()
@@ -441,7 +534,7 @@ class RLC(Cpt):
 
         znet = '%sZ%s %s %s %s; %s' % (self.namespace, self.relname, 
                                        self.relnodes[0], dummy_node,
-                                       arg_format(self.Zs(var)), 
+                                       arg_format(self.Z(var)), 
                                        opts)
 
         # Strip voltage and current labels from voltage source.
@@ -479,7 +572,7 @@ class RC(RLC):
 
         opts = self.opts.copy()
 
-        rnet = '%s %s %s %s; %s' % (self.name, 
+        rnet = '%s %s %s %s; %s' % (self.defname, 
                                     self.relnodes[0], dummy_node,
                                     arg_format(self.args[0]),
                                     opts)
@@ -510,19 +603,26 @@ class RC(RLC):
         if n2 >= 0:
             cct._G[n2, n2] += Y
 
-        if cct.kind == 'ivp' and self.cpt.hasic and n1 >= 0:
-            I = self.Isc.expr            
-            cct._Is[n1] += I
+        if cct.kind == 'ivp' and self.cpt.hasic:
+            I = self.Isc.expr
+            if n1 >= 0:
+                cct._Is[n1] += I 
+            if n2 >= 0:
+                cct._Is[n2] -= I               
 
 
 class C(RC):
 
     reactive = True
+
+    @property
+    def C(self):
+        return self.cpt.C
     
-    def kill_initial(self):
+    def kill(self):
         """Kill implicit sources due to initial conditions."""
         return '%s %s %s %s; %s' % (
-            self.name, self.relnodes[0], self.relnodes[1],
+            self.defname, self.relnodes[0], self.relnodes[1],
             arg_format(self.args[0]), self.opts)
 
     def pre_initial_model(self):
@@ -537,13 +637,28 @@ class C(RC):
     def ss_model(self):
         # Perhaps mangle name to ensure it does not conflict
         # with another voltage source?
-        return '%sV%s %s %s; %s' % (self.namespace, self.relname,
-                                    self.relnodes[0], self.relnodes[1], 
-                                    self.opts)
+        return '%sV_%s %s %s; %s' % (self.namespace, self.relname,
+                                     self.relnodes[0], self.relnodes[1], 
+                                     self.opts)
         
-    
+    @property
+    def V0(self):
+        """Initial voltage (for capacitors only)."""
 
-class E(DependentSource):
+        if self.cct.kind == 'ivp' and self.cpt.hasic:
+            return self.cpt.v0 / s
+        return 0
+
+
+class CPE(RC):
+
+    # If n == 0, then not reactive
+    reactive = True
+
+    pass
+
+
+class VCVS(DependentSource):
     """VCVS"""
 
     need_branch_current = True
@@ -574,7 +689,7 @@ class E(DependentSource):
         return '%sW %s %s; %s' % (self.namespace, self.relnodes[0],
                                   self.relnodes[1], newopts)
             
-class F(DependentSource):
+class CCCS(DependentSource):
     """CCCS"""
 
     need_control_current = True
@@ -602,7 +717,7 @@ class FB(Misc):
     pass
 
 
-class G(DependentSource):
+class VCCS(DependentSource):
     """VCCS"""
 
     def stamp(self, cct):
@@ -635,7 +750,7 @@ class GY(Dummy):
     def stamp(self, cct):
         
         n1, n2, n3, n4 = self.node_indexes
-        m1 = self.cct._branch_index(self.name + 'X')
+        m1 = self.cct._branch_index(self.defname + 'X')
         m2 = self.branch_index
 
         # m1 is the input branch
@@ -669,7 +784,7 @@ class GY(Dummy):
         cct._D[m2, m2] -= Z1
 
 
-class H(DependentSource):
+class CCVS(DependentSource):
     """CCVS"""
 
     need_branch_current = True
@@ -704,7 +819,7 @@ class I(IndependentSource):
         """Select domain kind for component."""
 
         return '%s %s %s %s; %s' % (
-            self.name, self.relnodes[0], self.relnodes[1],
+            self.defname, self.relnodes[0], self.relnodes[1],
             self.cpt.Isc.netval(kind), self.opts)
 
     def kill(self):
@@ -734,7 +849,7 @@ class I(IndependentSource):
 
     def s_model(self, var):
 
-        return '%s %s %s s %s; %s' % (self.name, 
+        return '%s %s %s s %s; %s' % (self.defname, 
                                       self.nodes[0], self.nodes[1],
                                       arg_format(self.Isc.laplace()(var)),
                                       self.opts)
@@ -748,17 +863,14 @@ class I(IndependentSource):
 
 class K(Dummy):
     
-    def __init__(self, cct, namespace, name, cpt_type, cpt_id, string,
+    def __init__(self, cct, namespace, defname, name, cpt_type, cpt_id, string,
                  opts_string, nodes, keyword, *args):
 
         self.Lname1 = args[0]
         self.Lname2 = args[1]
-        super (K, self).__init__(cct, namespace, name, cpt_type, cpt_id, string,
+        super (K, self).__init__(cct, namespace, defname, name,
+                                 cpt_type, cpt_id, string,
                                  opts_string, nodes, keyword, *args)
-        if len(args) > 2:
-            self.K = args[2]
-        else:
-            self.K = self.type + self.id
 
     def stamp(self, cct):
 
@@ -771,17 +883,17 @@ class K(Dummy):
 
         L1 = self.Lname1
         L2 = self.Lname2
+        K = self.cpt.K
 
-        ZL1 = cct.elements[L1].Z
-        ZL2 = cct.elements[L2].Z
-
-        ZM = self.K * sqrt(ZL1 * ZL2).simplify()
+        ZL1 = cct.elements[L1].Z.expr
+        ZL2 = cct.elements[L2].Z.expr
+        ZM = K.expr * sym.sqrt(ZL1 * ZL2)
 
         m1 = cct._branch_index(L1)
         m2 = cct._branch_index(L2)
 
-        cct._D[m1, m2] += -ZM.expr
-        cct._D[m2, m1] += -ZM.expr
+        cct._D[m1, m2] += -ZM
+        cct._D[m2, m1] += -ZM
 
 
 class L(RLC):
@@ -789,10 +901,22 @@ class L(RLC):
     need_branch_current = True
     reactive = True
 
-    def kill_initial(self):
+    @property
+    def I0(self):
+        """Initial current (for capacitors only)."""
+
+        if self.cct.kind == 'ivp' and self.cpt.hasic:
+            return self.cpt.i0 / s
+        return 0
+    
+    @property
+    def L(self):
+        return self.cpt.L
+    
+    def kill(self):
         """Kill implicit sources due to initial conditions."""
         return '%s %s %s %s; %s' % (
-            self.name, self.relnodes[0], self.relnodes[1],
+            self.defname, self.relnodes[0], self.relnodes[1],
             arg_format(self.args[0]), self.opts)
 
     def stamp(self, cct):
@@ -823,9 +947,9 @@ class L(RLC):
     def ss_model(self):
         # Perhaps mangle name to ensure it does not conflict
         # with another current source?        
-        return '%sI%s %s %s; %s' % (self.namespace, self.relname,
-                                    self.relnodes[0], self.relnodes[1], 
-                                    self.opts)
+        return '%sI_%s %s %s {-i_%s(t)}; %s' % (self.namespace, self.relname,
+                                                self.relnodes[0], self.relnodes[1],
+                                                self.relname, self.opts)
     
     def pre_initial_model(self):
 
@@ -842,6 +966,14 @@ class O(Dummy):
     def stamp(self, cct):
         pass
 
+    @property
+    def I(self):
+        return Current(0)
+
+    @property
+    def i(self):
+        return Current(0)(t)
+    
 
 class P(O):
     """Port"""
@@ -852,6 +984,12 @@ class R(RC):
     pass
 
 
+class RV(RC):
+
+    # TODO.  Can simulate as series resistors (1 - alpha) R and alpha R. 
+    pass
+
+
 class SPpp(Dummy):
 
     need_branch_current = True
@@ -987,6 +1125,12 @@ class TL(Misc):
     pass
 
 
+class Cable(Misc):
+    """Cable"""
+
+    pass
+
+
 class TP(Misc):
     """Two port"""
 
@@ -1017,12 +1161,13 @@ class TR(Dummy):
 class V(IndependentSource):
 
     need_branch_current = True
+    flip_branch_current = True        
 
     def select(self, kind=None):
         """Select domain kind for component."""
 
         return '%s %s %s %s; %s' % (
-            self.name, self.relnodes[0], self.relnodes[1],
+            self.defname, self.relnodes[0], self.relnodes[1],
             self.cpt.Voc.netval(kind), self.opts)        
 
     def kill(self):
@@ -1056,7 +1201,7 @@ class V(IndependentSource):
 
     def s_model(self, var):
 
-        return '%s %s %s s %s; %s' % (self.name, 
+        return '%s %s %s s %s; %s' % (self.defname, 
                                       self.relnodes[0], self.relnodes[1],
                                       arg_format(self.cpt.Voc.laplace()(var)),
                                       self.opts)
@@ -1074,6 +1219,14 @@ class W(Dummy):
     def stamp(self, cct):
         pass
 
+    @property
+    def I(self):
+        raise ValueError('Cannot determine current through wire, use a 0 V voltage source')
+
+    @property
+    def i(self):
+        raise ValueError('Cannot determine current through wire, use a 0 V voltage source')    
+    
 
 class XT(Misc):
     """Crystal"""
@@ -1111,20 +1264,15 @@ def make(classname, parent, name, cpt_type, cpt_id,
     # Create instance of component object
     newclass = classes[classname]
 
-    # Switch context
-    parent.context.switch()
-
     cpt = newclass(parent, name, cpt_type, cpt_id, string, opts_string, 
                    nodes, *args)
     # Add named attributes for the args?   Lname1, etc.
 
-    # Restore context
-    parent.context.restore()
-        
     return cpt
 
 
 # Dynamically create classes.
+defcpt('A', Misc, 'Annotation')
 defcpt('ADC', Misc, 'ADC')
 defcpt('AM', W, 'Ammeter')
 
@@ -1138,8 +1286,16 @@ defcpt('Dschottky', 'D', 'Schottky diode')
 defcpt('Dtunnel', 'D', 'Tunnel diode')
 defcpt('Dzener', 'D', 'Zener diode')
 
-defcpt('Eopamp', E, 'Opamp')
-defcpt('Efdopamp', E, 'Fully differential opamp')
+defcpt('E', VCVS, 'VCVS')
+defcpt('Eopamp', VCVS, 'Opamp')
+defcpt('Efdopamp', VCVS, 'Fully differential opamp')
+
+defcpt('F', CCCS, 'CCCS')
+defcpt('FS', Misc, 'Fuse')
+
+defcpt('G', VCCS, 'VCCS')
+
+defcpt('H', CCVS, 'CCVS')
 
 defcpt('sI', I, 's-domain current source')
 defcpt('Isin', I, 'Sinusoidal current source')
@@ -1194,6 +1350,11 @@ defcpt('Uchip2222', Logic, 'General purpose chip')
 defcpt('Uchip3131', Logic, 'General purpose chip')
 defcpt('Uchip3333', Logic, 'General purpose chip')
 defcpt('Uchip4141', Logic, 'General purpose chip')
+defcpt('Uchip4444', Logic, 'General purpose chip')
+defcpt('Uchip8181', Logic, 'General purpose chip')
+defcpt('Uchip8888', Logic, 'General purpose chip')
+defcpt('Umux21', Logic, '2-1 multiplexer')
+defcpt('Umux42', Logic, '4-2 multiplexer')
 defcpt('Udff', Misc, 'D flip-flop')
 defcpt('Ujkff', Misc, 'JK flip-flop')
 defcpt('Urslatch', Misc, 'RS latch')
@@ -1204,7 +1365,6 @@ defcpt('Vdc', V, 'DC voltage source')
 defcpt('Vstep', V, 'Step voltage source')
 defcpt('Vac', V, 'AC voltage source')
 defcpt('Vnoise', V, 'Noise voltage source')
-
 defcpt('VM', O, 'Voltmeter')
 
 # Append classes defined in this module but not imported.
