@@ -1,7 +1,7 @@
 """This module provides support for the common aspects of Circuit and
 Network classes.
 
-Copyright 2014--2019 Michael Hayes, UCECE
+Copyright 2014--2020 Michael Hayes, UCECE
 
 """
 
@@ -12,19 +12,24 @@ Copyright 2014--2019 Michael Hayes, UCECE
 from __future__ import division
 from .sexpr import Hs, Zs, Ys
 from .symbols import j, s, omega
-from .context import global_context
-from .super import Vsuper, Isuper, Vname, Iname
-from .schematic import Schematic, Opts, SchematicOpts
-from .mna import MNA, Nodedict, Branchdict
+from .voltage import Voltage, Vname
+from .current import Current, Iname
+from .schematic import Schematic, Opts
+from .mna import MNAMixin, Nodedict, Branchdict
 from .statespace import StateSpace
 from .netfile import NetfileMixin
 from .expr import Expr
+from .state import state
+from .attrdict import AttrDict
+from .immitance import ImmitanceMixin
+from .impedance import Impedance
+from .admittance import Admittance
 from . import mnacpts
 from copy import copy
 from collections import OrderedDict
 
 
-class Node(object):
+class Node(ImmitanceMixin):
 
     def __init__(self, cct, name):
 
@@ -50,6 +55,18 @@ class Node(object):
 
         return self.cct.get_vd(self.name, '0')
 
+    @property
+    def admittance(self):
+        """Driving-point admittance between node and ground."""
+
+        return self.cct.admittance(self.name, '0')
+
+    @property
+    def impedance(self):
+        """Driving-point impedance between node and ground."""        
+
+        return self.cct.impedance(self.name, '0')        
+
     def append(self, cpt):
 
         if cpt.type in ('P', ):
@@ -57,6 +74,24 @@ class Node(object):
 
         self.list.append(cpt)
 
+    def oneport(self, node=0):
+        """Create oneport object with respect to specified node
+        (default ground)."""
+
+        return self.cct.oneport(self.name, node)        
+        
+    def thevenin(self, node=0):
+        """Create Thevenin oneport object with respect to specified
+        node (default ground)."""
+        
+        return self.cct.thevenin(self.name, node)
+
+    def norton(self, node=0):
+        """Create Norton oneport object with respect to specified node
+        (default ground)."""
+        
+        return self.cct.norton(self.name, node)    
+        
 
 class NetlistNamespace(object):
     """This class allows elements, nodes, or other namespaces
@@ -155,11 +190,13 @@ class NetlistNamespace(object):
         kwargs include:
            label_ids: True to show component ids
            label_values: True to display component values
-           draw_nodes: True to show all nodes, False to show no nodes, 
+           draw_nodes: True to show all nodes,
+             False or 'none' to show no nodes, 
              'primary' to show primary nodes,
              'connections' to show nodes that connect more than two components,
              'all' to show all nodes
-           label_nodes: True to label all nodes, False to label no nodes, 
+           label_nodes: True to label all nodes,
+              False or 'none' to label no nodes, 
              'primary' to label primary nodes,
              'alpha' to label nodes starting with a letter,
              'pins' to label nodes that are pins on a chip,
@@ -168,7 +205,7 @@ class NetlistNamespace(object):
            scale: schematic scale factor, default 1.0
            node_spacing: spacing between component nodes, default 2.0
            cpt_size: size of a component, default 1.5
-           oversample: oversampling factor for png or pdf files
+           dpi: dots per inch for png files
            help_lines: distance between lines in grid, default 0.0 (disabled)
            debug: True to display debug information
         """
@@ -177,7 +214,7 @@ class NetlistNamespace(object):
         if kwargs.pop('s_model', False):
             cct = cct.s_model()
 
-        return cct.sch.draw(filename=filename, opts=self._netlist.opts, **kwargs)
+        return cct.sch.draw(filename=filename, **kwargs)
 
 
 class NetlistMixin(object):
@@ -186,15 +223,13 @@ class NetlistMixin(object):
 
         self._elements = OrderedDict()
         self.namespaces = {}
-        self.nodes = {}
+        self.nodes = AttrDict()
         if context is None:
-            context = global_context.new()
+            context = state.new_context()
         
         self.context = context
         self._init_parser(mnacpts)
 
-        self.opts = SchematicOpts()
-
         if filename is not None:
             self.netfile_add(filename)
 
@@ -234,6 +269,28 @@ class NetlistMixin(object):
         
         return self.netlist()
 
+    @property    
+    def pdb(self):
+        """Enter the python debugger."""
+        
+        import pdb; pdb.set_trace()
+        return self
+
+    @property
+    def symbols(self):
+        """Return dictionary of symbols defined in the circuit."""
+        
+        return self.context.symbols
+
+    @property
+    def all_symbols(self):
+        """Return dictionary of symbols defined in the circuit and global
+        symbols."""
+
+        symbols = self.symbols
+        symbols.update(state.global_context.symbols)
+        return symbols
+    
     @property
     def elements(self):
 
@@ -298,7 +355,6 @@ class NetlistMixin(object):
         """Create a copy of the netlist"""
 
         new = self._new()
-        new.opts = copy(self.opts)
 
         for cpt in self._elements.values():
             new._add(cpt.copy())
@@ -386,38 +442,68 @@ class NetlistMixin(object):
 
         self._node_map = node_map
         return node_map
+    
+    def augment_node_map(self, node_map={}):
+        """Create a mapping dict for all nodes."""
 
-    def create_node_map(self):
         # It would be desirable to renumber the nodes say from left to
         # right and top to bottom.  The schematic drawing algorithms
         # could help with this since they figure out the node
         # placement. 
-
-        enodes = self.equipotential_nodes
-        node_map = {}
         
-        count = 1        
+        enodes = self.equipotential_nodes
+
+        if '0' in self.nodes and '0' not in node_map:
+            node_map['0'] = '0'
+        
+        numbers = []
+        for m in range(len(enodes)):
+            numbers.append('%s' % (m + 1))
+        
+        for old, new in node_map.items():
+            if old not in self.nodes:
+                raise ValueError('Unknown node %s' % old)
+            if new in numbers:
+                numbers.remove(new)
+
+        # Ensure that enode keys are the nodes to be renamed.
+        enodes2 = {}
         for key, nodes in enodes.items():
-            snodes = sorted(nodes)
-            if snodes[0] == '0':
-                root = '0'
-            else:
-                root = '%d' % count            
-                count += 1
-            node_map[snodes[0]] = root
-            for m, enode in enumerate(snodes[1:]):
+            newkey = None
+            for node in nodes:
+                if node not in node_map:
+                    continue
+                if newkey is not None:
+                    # TODO, FIXME
+                    raise ValueError('Cannot rename two nodes of same potential')
+                newkey = node_map[node]
+            if newkey is None:
+                newkey = numbers.pop(0)
+            enodes2[key] = (newkey, nodes)
+
+        for key, foo in enodes2.items():
+
+            newkey, nodes = foo
+            
+            snodes = sorted(nodes.copy())
+
+            root = newkey
+            snodes.remove(key)
+            node_map[key] = root
+                
+            for m, enode in enumerate(snodes):
                 node_map[enode] = root + '_%d' % (m + 1)
+
         return node_map
     
-    def renumber(self, node_map=None):
+    def renumber(self, node_map={}):
         """Renumber nodes using specified node_map.  If node_map not specified
         then a mapping is created."""
 
-        if node_map is None:
-            node_map = self.create_node_map()
+        if len(node_map) != len(self.nodes):
+            node_map = self.augment_node_map(node_map)
 
         new = self._new()
-        new.opts = copy(self.opts)
 
         for cpt in self._elements.values():
             new._add(cpt.rename_nodes(node_map))
@@ -455,128 +541,173 @@ class NetlistMixin(object):
                 self._branch_list.append(elt.name)                
         return self._branch_list
 
-    def Voc(self, Np, Nm):
+    def _parse_node_args(self, Np, Nm=None):
+        
+        if Nm is None:
+            cpt = self[Np]
+            if isinstance(cpt, Node):
+                Np, Nm = cpt.name, 0
+            else:
+                Np, Nm = cpt.nodes[0:2]
+        return Np, Nm
+    
+    def Voc(self, Np, Nm=None):
         """Return open-circuit transform-domain voltage between nodes Np and
         Nm."""
 
         return self.get_Vd(Np, Nm)
 
-    def voc(self, Np, Nm):
+    def voc(self, Np, Nm=None):
         """Return open-circuit t-domain voltage between nodes Np and Nm."""
 
         return self.Voc(Np, Nm).time()
 
-    def Isc(self, Np, Nm):
+    def Isc(self, Np, Nm=None):
         """Return short-circuit transform-domain current between nodes Np and
         Nm."""
 
+        Np, Nm = self._parse_node_args(Np, Nm)
+        
         new = self.copy()
-        new.add('Vshort_ %d %d 0' % (Np, Nm))
-        Isc = new.Vshort_.I
-        new.remove('Vshort_')
+        if new.is_causal:
+            new.add('Vshort_ %s %s step 0' % (Np, Nm))
+        else:
+            new.add('Vshort_ %s %s 0' % (Np, Nm))            
 
+        # Negate current since Vshort is a considered a source.
+        Isc = -new.Vshort_.I
+        new.remove('Vshort_')
+        
         return Isc
 
-    def isc(self, Np, Nm):
+    def isc(self, Np, Nm=None):
         """Return short-circuit t-domain current between nodes Np and Nm."""
 
         return self.Isc(Np, Nm).time()
 
-    def thevenin(self, Np, Nm):
-        """Return s-domain Thevenin model between nodes Np and Nm."""
+    def oneport(self, Np, Nm=None):
+        """Return oneport object between nodes Np and Nm.  This might be a
+        Thevenin network, a Norton network, or a single component.
+
+        If Np is a component name, create model using the component nodes."""
+
+        try:
+            return self.norton(Np, Nm).simplify()
+        except:
+            return self.thevenin(Np, Nm).simplify()
+    
+    def thevenin(self, Np, Nm=None):
+        """Return s-domain Thevenin oneport model between nodes Np and Nm.
+
+        If Np is a component name, create model using the component nodes."""
 
         from .oneport import V, Z
 
-        Voc = self.Voc(Np, Nm).laplace()
+        Np, Nm = self._parse_node_args(Np, Nm)
+        Voc = self.Voc(Np, Nm)
         Zoc = self.impedance(Np, Nm)
 
-        return V(Voc) + Z(Zoc)
+        # Convert to time-domain to handle arbitrary sources.  Either
+        # this or define a way to represent a superposition in a
+        # netlist.
+        return V(Voc.time()) + Z(Zoc)
 
-    def norton(self, Np, Nm):
-        """Return s-domain Norton model between nodes Np and Nm."""
+    def norton(self, Np, Nm=None):
+        """Return s-domain Norton model between nodes Np and Nm.
+
+        If Np is a component name, create model using the component nodes."""
 
         from .oneport import I, Y
 
-        Isc = self.Isc(Np, Nm).laplace()
+        Np, Nm = self._parse_node_args(Np, Nm)
+        Isc = self.Isc(Np, Nm)
         Ysc = self.admittance(Np, Nm)
+
+        # Convert to time-domain to handle arbitrary sources.  Either
+        # this or define a way to represent a superposition in a
+        # netlist.        
+        return I(Isc.time()) | Y(Ysc)
+
+    def admittance(self, Np, Nm=None):
+        """Return driving-point admittance between nodes
+        Np and Nm with independent sources killed and initial
+        conditions ignored.  Since the result is causal, the frequency
+        domain admittance can be found by substituting j * omega for
+        s."""        
+
+        Np, Nm = self._parse_node_args(Np, Nm)
         
-        return I(Isc) | Y(Ysc)
-
-    def admittance(self, Np, Nm):
-        """Return s-domain admittance between nodes Np and Nm with independent 
-        sources killed.
-
-        """
-
         new = self.kill()
+        if '0' not in new.nodes:
+            new.add('W %s 0' % Nm)        
 
         # Connect 1 V s-domain voltage source between nodes and
         # measure current.
-        new._add('Vin_ %d %d {DiracDelta(t)}' % (Np, Nm))
-        If = -new.Vin_.I
+        new._add('Vin_ %s %s {DiracDelta(t)}' % (Np, Nm))
+        If = new.Vin_.I
         new.remove('Vin_')
 
-        return Ys(If.laplace(), causal=True)
+        return Admittance(If.laplace(), kind=self.kind)
 
-    def impedance(self, Np, Nm):
-        """Return s-domain impedance between nodes Np and Nm with independent
-        sources killed.
-
-        """
+    def impedance(self, Np, Nm=None):
+        """Return driving-point impedance between nodes
+        Np and Nm with independent sources killed and initial
+        conditions ignored.  Since the result is causal, the frequency
+        domain impedance can be found by substituting j * omega for
+        s."""
 
+        Np, Nm = self._parse_node_args(Np, Nm)
+        
         new = self.kill()
+        if '0' not in new.nodes:
+            new.add('W %s 0' % Nm)
 
         # Connect 1 A s-domain current source between nodes and
         # measure voltage.
-        new._add('Iin_ %d %d {DiracDelta(t)}' % (Np, Nm))
+        new._add('Iin_ %s %s {DiracDelta(t)}' % (Np, Nm))
         Vf = new.Voc(Np, Nm)
         new.remove('Iin_')
 
-        return Zs(Vf.laplace(), causal=True)
+        return Impedance(Vf.laplace(), kind=self.kind)
 
-    def resistance(self, Np, Nm):
+    def resistance(self, Np, Nm=None):
         """Return resistance between nodes Np and Nm with independent
         sources killed.  The result is in the AC (omega) domain.
+        See also conductance, reactance, susceptance."""
+        return self.impedance(Np, Nm).R
 
-        """
-        return self.impedance(Np, Nm).jomega.real
-
-    def reactance(self, Np, Nm):
+    def reactance(self, Np, Nm=None):
         """Return reactance between nodes Np and Nm with independent
         sources killed.  The result is in the AC (omega) domain.
+        See also conductance, resistance, susceptance."""
+        return self.impedance(Np, Nm).X
 
-        """
-        return self.impedance(Np, Nm).jomega.imag * j
-
-    def conductance(self, Np, Nm):
+    def conductance(self, Np, Nm=None):
         """Return conductance (inverse resistance) between nodes Np and Nm
         with independent sources killed.  The result is in the AC (omega)
-        domain.
+        domain.    See also resistance, reactance, susceptance."""
+        return self.impedance(Np, Nm).G
 
-        """
-        return 1 / self.resistance
-
-    def susceptance(self, Np, Nm):
+    def susceptance(self, Np, Nm=None):
         """Return susceptance (inverse reactance) between nodes Np and Nm with
         independent sources killed.  The result is in the AC (omega)
-        domain.
-
-        """
-        return 1 / self.reactance
+        domain.   See also conductance, reactance, resistance."""
+        return self.impedance(Np, Nm).B
         
     def transfer(self, N1p, N1m, N2p, N2m):
-        """Create s-domain voltage transfer function V2 / V1 where:
+        """Create s-domain voltage transfer function V2(s) / V1(s) where:
         V1 is V[N1p] - V[N1m]
         V2 is V[N2p] - V[N2m]
 
-        Note, independent sources are killed.
-
-        """
-
-        # TODO, work with AC models
+        Note, independent sources are killed and initial conditions
+        are ignored.  Since the result is causal, the frequency response
+        can be found by substituting j * omega for s."""
 
         new = self.kill()
-        new._add('V1_ %d %d {DiracDelta(t)}' % (N1p, N1m))
+        if '0' not in new.nodes:
+            new.add('W %s 0' % Nm)
+        
+        new._add('V1_ %s %s {DiracDelta(t)}' % (N1p, N1m))
 
         V2 = new.Voc(N2p, N2m)
         V1 = new.V1_.V
@@ -597,7 +728,7 @@ class NetlistMixin(object):
             raise ValueError('Network contains independent sources')
 
         try:
-            self.add('V1_ %d %d {DiracDelta(t)}' % (N1p, N1m))
+            self.add('V1_ %s %s {DiracDelta(t)}' % (N1p, N1m))
 
             # A11 = V1 / V2 with I2 = 0
             # Apply V1 and measure V2 with port 2 open-circuit
@@ -609,7 +740,7 @@ class NetlistMixin(object):
 
             self.remove('V1_')
 
-            self.add('I1_ %d %d {DiracDelta(t)}' % (N1p, N1m))
+            self.add('I1_ %s %s {DiracDelta(t)}' % (N1p, N1m))
 
             # A21 = I1 / V2 with I2 = 0
             # Apply I1 and measure I2 with port 2 open-circuit
@@ -633,13 +764,10 @@ class NetlistMixin(object):
         """
 
         new = self._new()
-        new.opts = copy(self.opts)
 
         for cpt in self._elements.values():
             if cpt.independent_source:
                 net = cpt.select(kind)                
-            elif kind != 'ivp':
-                net = cpt.kill_initial()
             else:
                 net = cpt.copy()
             new._add(net)
@@ -648,17 +776,15 @@ class NetlistMixin(object):
     def _kill(self, sourcenames):
 
         new = self._new()
-        new.opts = copy(self.opts)
 
         for cpt in self._elements.values():
-            if cpt.name in self.control_sources:
-                net = cpt.zero()                
-            elif cpt.name in sourcenames:
-                net = cpt.kill()
-            elif 'ICs' in sourcenames:
-                net = cpt.kill_initial()
+            if cpt.name in sourcenames:
+                if cpt.name in self.control_sources:
+                    net = cpt.zero()                
+                else:
+                    net = cpt.kill()
             else:
-                net = cpt.copy()
+                net = cpt.copy()                
             new._add(net)
         return new        
 
@@ -708,7 +834,6 @@ class NetlistMixin(object):
     def _noisy(self, resistornames):
 
         new = self._new()
-        new.opts = copy(self.opts)
 
         for cpt in self._elements.values():
             if cpt.name in resistornames:
@@ -794,7 +919,6 @@ class NetlistMixin(object):
         conditions."""
 
         new = self._new()
-        new.opts = copy(self.opts)
 
         for cpt in self._elements.values():
             net = cpt.pre_initial_model()
@@ -805,7 +929,6 @@ class NetlistMixin(object):
         """"Create s-domain model."""
 
         new = self._new()
-        new.opts = copy(self.opts)
 
         for cpt in self._elements.values():
             net = cpt.s_model(var)
@@ -817,7 +940,6 @@ class NetlistMixin(object):
         with current sources and capacitors with voltage sources."""
 
         new = self._new()
-        new.opts = copy(self.opts)
 
         for cpt in self._elements.values():
             net = cpt.ss_model()
@@ -861,7 +983,7 @@ class NetlistMixin(object):
            scale: schematic scale factor, default 1.0
            node_spacing: spacing between component nodes, default 2.0
            cpt_size: size of a component, default 1.5
-           oversample: oversampling factor for png or pdf files
+           dpi: dots per inch for png files
            help_lines: distance between lines in grid, default 0.0 (disabled)
            debug: True to display debug information
         """
@@ -870,7 +992,7 @@ class NetlistMixin(object):
         if kwargs.pop('s_model', False):
             cct = cct.s_model()
 
-        return cct.sch.draw(filename=filename, opts=self.opts, **kwargs)
+        return cct.sch.draw(filename=filename, **kwargs)
 
     @property
     def is_causal(self):
@@ -891,11 +1013,26 @@ class NetlistMixin(object):
         initial value problem."""
         return self.analysis['ac']
 
+    @property
+    def has_dc(self):
+        """Return True if any independent source has a DC component."""
+        return self.analysis['has_dc']
+
+    @property
+    def has_ac(self):
+        """Return True if any independent source has an AC component."""
+        return self.analysis['has_ac']
+
     @property
     def has_s(self):
         """Return True if any independent source has an s-domain component."""
-        return self.analysis['has_s']
+        return self.analysis['has_s']    
 
+    @property    
+    def has_transient(self):
+        """Return True if any independent source has a transient component."""
+        return self.analysis['has_transient']    
+    
     @property
     def zeroic(self):
         """Return True if the initial conditions for all components are zero."""
@@ -997,6 +1134,7 @@ class NetlistMixin(object):
         hasic = False
         zeroic = True
         has_s = False
+        has_transient = False        
         ac_count = 0
         dc_count = 0
         causal = True
@@ -1016,6 +1154,8 @@ class NetlistMixin(object):
                 independent_sources.append(key)
                 if elt.has_s:
                     has_s = True
+                if elt.has_transient:
+                    has_transient = True                    
                 if elt.is_ac:
                     ac_count += 1
                 if elt.is_dc:
@@ -1031,9 +1171,12 @@ class NetlistMixin(object):
                     
         analysis = {} 
         analysis['zeroic'] = zeroic
-        analysis['hasic'] = hasic
+        analysis['has_ic'] = hasic        
         analysis['ivp'] = hasic
+        analysis['has_dc'] = dc_count > 0
+        analysis['has_ac'] = ac_count > 0        
         analysis['has_s'] = has_s
+        analysis['has_transient'] = has_transient
         analysis['dependent_sources'] = dependent_sources        
         analysis['independent_sources'] = independent_sources
         analysis['control_sources'] = control_sources        
@@ -1204,7 +1347,7 @@ class Netlist(NetlistMixin, NetfileMixin):
         for sub in self.sub.values():
             for node, value in sub.Vdict.items():
                 if node not in result:
-                    result[node] = Vsuper()
+                    result[node] = Voltage()
                 result[node].add(value)
 
         self._Vdict = result
@@ -1223,7 +1366,7 @@ class Netlist(NetlistMixin, NetfileMixin):
         for sub in self.sub.values():
             for node, value in sub.Idict.items():
                 if node not in result:
-                    result[node] = Isuper()
+                    result[node] = Current()
                 result[node].add(value)
         self._Idict = result                    
         return result    
@@ -1231,7 +1374,7 @@ class Netlist(NetlistMixin, NetfileMixin):
     def get_I(self, name):
         """Current through component"""
 
-        result = Isuper()
+        result = Current()
         for sub in self.sub.values():
             I = sub.get_I(name)
             result.add(I)
@@ -1243,7 +1386,7 @@ class Netlist(NetlistMixin, NetfileMixin):
 
         return self.get_I(name).time()
 
-    def get_Vd(self, Np, Nm):
+    def get_Vd(self, Np, Nm=None):
         """Voltage drop between nodes"""
 
         if isinstance(Nm, int):
@@ -1251,14 +1394,14 @@ class Netlist(NetlistMixin, NetfileMixin):
         if isinstance(Np, int):
             Np = '%s' % Np            
 
-        result = Vsuper()
+        result = Voltage()
         for sub in self.sub.values():
             Vd = sub.get_Vd(Np, Nm)
             result.add(Vd)
         result = result.canonical()
         return result
 
-    def get_vd(self, Np, Nm):
+    def get_vd(self, Np, Nm=None):
         """Time-domain voltage drop between nodes"""
 
         return self.get_Vd(Np, Nm).time()
@@ -1301,7 +1444,7 @@ class Netlist(NetlistMixin, NetfileMixin):
         return SubNetlist(self, 'laplace')    
     
     
-class SubNetlist(NetlistMixin, MNA):
+class SubNetlist(NetlistMixin, MNAMixin):
     """This is a representation of a netlist for a particular
     transformation domain, such as ac, dc, transient, or noise."""
 
@@ -1309,7 +1452,7 @@ class SubNetlist(NetlistMixin, MNA):
 
         obj = netlist.select(kind=kind)
         # Need own context to avoid conflicts with Vn1 and Vn1(s), etc.
-        obj.context = global_context.new()        
+        obj.context = state.new_context()
         obj.kind = kind
         obj.__class__ = cls
         obj._analysis = obj.analyse()
@@ -1341,13 +1484,13 @@ class SubNetlist(NetlistMixin, MNA):
 
         return self.get_I(name).time()
 
-    def get_Vd(self, Np, Nm):
+    def get_Vd(self, Np, Nm=None):
         """Voltage drop between nodes"""
 
         self._solve()
         return (self._Vdict[Np] - self._Vdict[Nm]).canonical()
 
-    def get_vd(self, Np, Nm):
+    def get_vd(self, Np, Nm=None):
         """Time-domain voltage drop between nodes"""
 
         return self.get_Vd(Np, Nm).time()
