@@ -1,6 +1,5 @@
-"""
-This module performs nodal analysis.  It is primarily for showing
-the equations rather than evaluating them.
+"""This module performs nodal analysis.  It is primarily for showing
+the equations rather than for evaluating them.
 
 Copyright 2019 Michael Hayes, UCECE
 
@@ -39,9 +38,19 @@ class NodalAnalysis(object):
     ... W 2 6; up
     ... C1 5 6; right=2
     ...''')    
-    >>>
-    >>> na = NodalAnalysis(cct.sub['s'])
+
+    To perform nodal analysis in the Laplace domain:
+
+    >>> na = NodalAnalysis(cct.laplace)
+    
+    To display the system of equations (in matrix form) that needs to
+    be solved:
+    
     >>> na.equations().pprint()
+    
+    To display the equations found by applying KCL at each node:
+    
+    >>> na.nodal_equations().pprint()
 
     """
 
@@ -98,7 +107,7 @@ class NodalAnalysis(object):
                 n1 = self.G.node_map[elt.nodes[0]]
                 n2 = self.G.node_map[elt.nodes[1]]
 
-                V = Vsuper(elt.voc).select(self.cct.kind)
+                V = Voltage(elt.voc).select(self.cct.kind)
                 
                 eq = Eq(self.ydict[n1], self.ydict[n2] + V)
 
@@ -140,8 +149,7 @@ class NodalAnalysis(object):
         
         return expr(Eq(sym.MatMul(self.A, self.y), self.b))
 
-
 from .expr import Expr, ExprDict, expr
 from .texpr import Vt
-from .super import Vsuper
+from .voltage import Voltage
 from .matrix import matrix
