@@ -1,10 +1,17 @@
+"""This file provides the omegaExpr class to represent omega-domain
+(angular frequency Fourier domain) expressions.
+
+Copyright 2014--2019 Michael Hayes, UCECE
+
+"""
+
 from __future__ import division
 from .fourier import inverse_fourier_transform
-from .sfwexpr import sfwExpr
+from .expr import Expr
 from .sym import fsym, ssym, tsym, omegasym, j, pi
 
 
-class omegaExpr(sfwExpr):
+class omegaExpr(Expr):
 
     """Fourier domain expression or symbol (angular frequency)."""
 
@@ -159,6 +166,15 @@ class Homega(omegaExpr):
         super(Homega, self).__init__(val, **assumptions)
         self._fourier_conjugate_class = Ht
 
+        
+def omegaexpr(arg):
+    """Create omegaExpr object.  If `arg` is omegasym return omega"""
+
+    if arg is omegasym:
+        return omega
+    return omegaExpr(arg)
+
+        
 from .texpr import Ht, It, Vt, Yt, Zt, tExpr
 omega = omegaExpr('omega')
 
