@@ -20,7 +20,7 @@ Copyright 2014--2019 Michael Hayes, UCECE
 
 # TODO.   Rethink best way to handle impedances.   These can either be
 # s-domain or omega domain (or DC as a special case where omega = 0).
-# Perhaps alays use Isuper and Vsuper and have these classes
+# Perhaps alays use Current and Voltage and have these classes
 # handle multiplication/division with admittances/impedances?
 
 
@@ -29,12 +29,15 @@ from __future__ import division
 from .functions import Heaviside, cos, exp
 from .symbols import j, t, s
 from .network import Network
+from .immitance import ImmitanceMixin
+from .impedance import Impedance
+from .admittance import Admittance
 
 
 __all__ = ('V', 'I', 'v', 'i', 'R', 'L', 'C', 'G', 'Y', 'Z',
            'Vdc', 'Vstep', 'Idc', 'Istep', 'Vac', 'sV', 'sI',
            'Iac', 'Vnoise', 'Inoise', 
-           'Par', 'Ser', 'Xtal', 'FerriteBead')
+           'Par', 'Ser', 'Xtal', 'FerriteBead', 'CPE')
 
 def _check_oneport_args(args):
 
@@ -43,7 +46,7 @@ def _check_oneport_args(args):
             raise ValueError('%s not a OnePort' % arg1)
 
 
-class OnePort(Network):
+class OnePort(Network, ImmitanceMixin):
     """One-port network
 
     There are four major types of OnePort:
@@ -76,22 +79,22 @@ class OnePort(Network):
     _Isc = None
 
     @property
-    def Z(self):
+    def impedance(self):
         if self._Z is not None:
             return self._Z
         if self._Y is not None:
-            return Zs(1 / self._Y)
+            return Impedance(1 / self._Y)
         if self._Voc is not None:        
-            return Zs(0)
+            return Impedance(0)
         if self._Isc is not None:        
-            return Zs(1 / Ys(0))
+            return Impedance(1 / Admittance(0))
         raise ValueError('_Isc, _Voc, _Y, or _Z undefined for %s' % self)
 
     @property
-    def Y(self):
+    def admittance(self):
         if self._Y is not None:
             return self._Y
-        return Ys(1 / self.Z)
+        return Admittance(1 / self.impedance)
 
     @property
     def Voc(self):
@@ -99,11 +102,11 @@ class OnePort(Network):
         if self._Voc is not None:
             return self._Voc
         if self._Isc is not None:
-            return self._Isc * self.Z
+            return self._Isc * self.impedance
         if self._Z is not None:        
-            return Vsuper(0)
+            return Voltage(0)
         if self._Y is not None:        
-            return Isuper(0)
+            return Current(0)
         raise ValueError('_Isc, _Voc, _Y, or _Z undefined for %s' % self)
     
     @property
@@ -111,7 +114,7 @@ class OnePort(Network):
         """Short-circuit current."""        
         if self._Isc is not None:
             return self._Isc
-        return self.Voc / self.Z
+        return self.Voc / self.impedance
 
     @property
     def V(self):
@@ -121,14 +124,14 @@ class OnePort(Network):
     @property
     def I(self):
         """Open-circuit current.  Except for a current source this is zero."""
-        return Isuper(0)
+        return Current(0)
 
     @property
     def i(self):
         """Open-circuit time-domain current.  Except for a current source this
         is zero."""
         return self.I.time()
-    
+
     def __add__(self, OP):
         """Series combination"""
 
@@ -202,19 +205,19 @@ class OnePort(Network):
     @property
     def z(self):
         """Impedance impulse-response."""
-        return self.Z.time()
+        return self.impedance.time()
 
     @property
     def y(self):
         """Admittance impulse-response."""        
-        return self.Y.time()
+        return self.admittance.time()
 
     def thevenin(self):
         """Simplify to a Thevenin network"""
 
         new = self.simplify()
         Voc = new.Voc
-        Z = new.Z
+        Z = new.impedance
 
         if Voc.is_superposition and not Z.is_real:
             print('Warning, detected superposition with reactive impedance,'
@@ -246,7 +249,7 @@ class OnePort(Network):
 
         new = self.simplify()
         Isc = new.Isc
-        Y = new.Y
+        Y = new.admittance
         
         if Isc.is_superposition and not Y.is_real:
             print('Warning, detected superposition with reactive impedance,'
@@ -278,18 +281,18 @@ class OnePort(Network):
 
         if self._Voc is not None:
             if self._Voc == 0:
-                return Z(self.Z)
+                return Z(self.impedance)
             Voc = self._Voc.laplace()
             if self.Z == 0:
                 return V(Voc)
-            return Ser(V(Voc), Z(self.Z))
+            return Ser(V(Voc), Z(self.impedance))
         elif self._Isc is not None:
             if self._Isc == 0:
-                return Y(self.Y)
+                return Y(self.admittance)
             Isc = self._Isc.laplace()
-            if self.Y == 0:
+            if self.admittance == 0:
                 return I(Isc)
-            return Par(I(Isc), Y(self.Y))
+            return Par(I(Isc), Y(self.admittance))
         elif self._Z is not None:
             return Z(self._Z)
         elif self._Y is not None:
@@ -304,7 +307,7 @@ class OnePort(Network):
         if not isinstance(self, (R, G, Y, Z)):
             return self
         
-        R1 = self.Z.real
+        R1 = self.R
         if R1 != 0:
             Vn = Vnoise('sqrt(4 * k * T * %s)' % R1(j * omega))
             return self + Vn
@@ -385,18 +388,18 @@ class ParSer(OnePort):
                     return arg2
                 if isinstance(arg2, V) and arg2.Voc == 0:
                     return arg1
-                if isinstance(arg1, Z) and arg1.Z == 0:
+                if isinstance(arg1, (R, Z)) and arg1.impedance == 0:
                     return arg2
-                if isinstance(arg2, Z) and arg2.Z == 0:
+                if isinstance(arg2, (R, Z)) and arg2.impedance == 0:
                     return arg1
             if self.__class__ == Par:
                 if isinstance(arg1, I) and arg1.Isc == 0:
                     return arg2
                 if isinstance(arg2, I) and arg2.Isc == 0:
                     return arg1
-                if isinstance(arg1, Y) and arg1.Y == 0:
+                if isinstance(arg1, (Y, G)) and arg1.admittance == 0:
                     return arg2
-                if isinstance(arg2, Y) and arg2.Y == 0:
+                if isinstance(arg2, (Y, G)) and arg2.admittance == 0:
                     return arg1
 
             return None
@@ -410,7 +413,7 @@ class ParSer(OnePort):
             if isinstance(arg1, V):
                 return V(arg1 + arg2)
             if isinstance(arg1, R):
-                return R(arg1.R + arg2.R)
+                return R(arg1._R + arg2._R)
             if isinstance(arg1, L):
                 # The currents should be the same!
                 if arg1.i0 != arg2.i0 or arg1.hasic != arg2.hasic:
@@ -419,7 +422,7 @@ class ParSer(OnePort):
                 i0 = arg1.i0 if arg1.hasic else None
                 return L(arg1.L + arg2.L, i0)
             if isinstance(arg1, G):
-                return G(arg1.G * arg2.G / (arg1.G + arg2.G))
+                return G(arg1._G * arg2._G / (arg1._G + arg2._G))
             if isinstance(arg1, C):
                 v0 = arg1.v0 + arg2.v0 if arg1.hasic or arg2.hasic else None
                 return C(arg1.C * arg2.C / (arg1.C + arg2.C), v0)
@@ -434,7 +437,7 @@ class ParSer(OnePort):
             if isinstance(arg1, I):
                 return I(arg1 + arg2)
             if isinstance(arg1, G):
-                return G(arg1.G + arg2.G)
+                return G(arg1._G + arg2._G)
             if isinstance(arg1, C):
                 # The voltages should be the same!
                 if arg1.v0 != arg2.v0 or arg1.hasic != arg2.hasic:
@@ -443,7 +446,7 @@ class ParSer(OnePort):
                 v0 = arg1.v0 if arg1.hasic else None
                 return C(arg1.C + arg2.C, v0)
             if isinstance(arg1, R):
-                return R(arg1.R * arg2.R / (arg1.R + arg2.R))
+                return R(arg1._R * arg2._R / (arg1._R + arg2._R))
             if isinstance(arg1, L):
                 i0 = arg1.i0 + arg2.i0 if arg1.hasic or arg2.hasic else None
                 return L(arg1.L * arg2.L / (arg1.L + arg2.L), i0)
@@ -549,15 +552,6 @@ class ParSer(OnePort):
     def Voc(self):
         return self.cct.Voc(1, 0)
 
-    @property
-    def Y(self):
-        # Could extract directly if have Y || I or Z + V
-        return self.cct.admittance(1, 0)
-
-    @property
-    def Z(self):
-        # Could extract directly if have Y || I or Z + V
-        return self.cct.impedance(1, 0)
 
 class Par(ParSer):
     """Parallel class"""
@@ -575,11 +569,12 @@ class Par(ParSer):
                 if isinstance(arg1, V) and isinstance(arg2, V):
                     raise ValueError('Voltage sources connected in parallel'
                                      ' %s and %s' % (arg1, arg2))
-                if isinstance(arg1, V) or isinstance(arg2, V):
-                    # Should simplify by removing redundant component to
-                    # save later grief with Norton or Thevenin transformation.
-                    print('Warn: parallel connection with voltage source:'
-                          ' %s and %s' % (arg1, arg2))
+                elif isinstance(arg1, V):
+                    print('Warn: redundant component %s in parallel with voltage source %s' % (arg2, arg1))
+
+                elif isinstance(arg2, V):
+                    print('Warn: redundant component %s in parallel with voltage source %s' % (arg1, arg2))
+
 
     @property
     def width(self):
@@ -655,6 +650,16 @@ class Par(ParSer):
         s.append('W %s %s; right=%s' % (n4, n2, self.wsep))
         return '\n'.join(s)
 
+    @property
+    def admittance(self):
+        Y = 0
+        for arg in self.args:
+            Y += arg.admittance
+        return Admittance(Y)
+
+    @property
+    def impedance(self):
+        return Impedance(1 / self.admittance)
 
 class Ser(ParSer):
     """Series class"""
@@ -672,11 +677,11 @@ class Ser(ParSer):
                 if isinstance(arg1, I) and isinstance(arg2, I):
                     raise ValueError('Current sources connected in series'
                                      ' %s and %s' % (arg1, arg2))
-                if isinstance(arg1, I) or isinstance(arg2, I):
-                    # Should simplify by removing redundant component to
-                    # save later grief with Norton or Thevenin transformation.
-                    print('Warn: series connection with current source:'
-                          ' %s and %s' % (arg1, arg2))
+                elif isinstance(arg1, I):
+                    print('Warn: redundant component %s in series with current source %s' % (arg2, arg1))
+
+                elif isinstance(arg2, I):
+                    print('Warn: redundant component %s in series with current source %s' % (arg1, arg2))                    
 
     @property
     def height(self):
@@ -712,15 +717,26 @@ class Ser(ParSer):
         s.append(self.args[-1].net_make(net, n1, n2))
         return '\n'.join(s)
 
+    @property
+    def Admittance(self):
+        return Admittance(1 / self.impedance)
+    
+    @property
+    def impedance(self):
+        Z = 0
+        for arg in self.args:
+            Z += arg.impedance
+        return Impedance(Z)
 
+    
 class R(OnePort):
     """Resistor"""
 
     def __init__(self, Rval):
 
         self.args = (Rval, )
-        self.R = cExpr(Rval)
-        self._Z = Zs.R(self.R)
+        self._R = cExpr(Rval)
+        self._Z = Impedance(self._R)
 
 
 class G(OnePort):
@@ -729,8 +745,8 @@ class G(OnePort):
     def __init__(self, Gval):
 
         self.args = (Gval, )
-        self.G = cExpr(Gval)
-        self._Z = 1 / Ys.G(self.G)
+        self._G = cExpr(Gval)
+        self._Z = 1 / Admittance(self._G)
 
     def net_make(self, net, n1=None, n2=None):
 
@@ -738,7 +754,7 @@ class G(OnePort):
             n1 = net.node
         if n2 == None:
             n2 = net.node
-        return 'R %s %s {%s}; right' % (n1, n2, 1 / self.G)
+        return 'R %s %s {%s}; right' % (n1, n2, 1 / self._G)
 
 
 class L(OnePort):
@@ -761,8 +777,8 @@ class L(OnePort):
         i0 = cExpr(i0)
         self.L = Lval
         self.i0 = i0
-        self._Z = Zs.L(Lval)
-        self._Voc = Vsuper(-Vs(i0 * Lval))
+        self._Z = Impedance(s * Lval)
+        self._Voc = Voltage(-Vs(i0 * Lval))
         self.zeroic = self.i0 == 0 
 
 
@@ -786,19 +802,45 @@ class C(OnePort):
         v0 = cExpr(v0)
         self.C = Cval
         self.v0 = v0
-        self._Z = Zs.C(Cval)
-        self._Voc = Vsuper(Vs(v0) / s)
+        self._Z = Impedance(1 / (s * Cval))
+        self._Voc = Voltage(Vs(v0) / s)
         self.zeroic = self.v0 == 0
 
 
+class CPE(OnePort):
+    """Constant phase element
+
+    This has an impedance 1 / (s**alpha * K).  When alpha == 0, the CPE is
+    equivalent to a resistor of resistance 1 / K.  When alpha == 1, the CPE is
+    equivalent to a capacitor of capacitance K.
+
+    When alpha == 0.5 (default), the CPE is a Warburg element.
+
+    The phase of the impedance is -pi * alpha / 2.
+
+    Note, when alpha is non-integral, the impedance cannot be represented
+    as a rational function and so there are no poles or zeros.  So
+    don't be suprised if Lcapy throws an occasional wobbly."""
+
+    def __init__(self, K, alpha=0.5):
+
+        self.args = (K, alpha)
+
+        K = cExpr(K)
+        alpha = cExpr(alpha)
+        self.K = K
+        self.alpha = alpha
+        self._Z = Impedance(1 / (s ** alpha * K))
+
+
 class Y(OnePort):
     """General admittance."""
 
     def __init__(self, Yval):
 
         self.args = (Yval, )
-        Yval = Ys(Yval, causal=True)
-        self._Z = 1 / Yval
+        Yval = Admittance(Yval)
+        self._Y = Yval
 
 
 class Z(OnePort):
@@ -807,7 +849,7 @@ class Z(OnePort):
     def __init__(self, Zval):
 
         self.args = (Zval, )
-        Zval = Zs(Zval, causal=True)
+        Zval = Impedance(Zval)
         self._Z = Zval
 
 
@@ -827,7 +869,7 @@ class sV(VoltageSource):
 
         self.args = (Vval, )
         Vval = sExpr(Vval)
-        self._Voc = Vsuper(Vs(Vval))
+        self._Voc = Voltage(Vs(Vval))
 
 
 class V(VoltageSource):
@@ -836,7 +878,7 @@ class V(VoltageSource):
     def __init__(self, Vval):
 
         self.args = (Vval, )
-        self._Voc = Vsuper(Vval)
+        self._Voc = Voltage(Vval)
 
         
 class Vstep(VoltageSource):
@@ -848,7 +890,7 @@ class Vstep(VoltageSource):
 
         self.args = (v, )
         v = cExpr(v)
-        self._Voc = Vsuper(tExpr(v) * Heaviside(t), causal=True)
+        self._Voc = Voltage(tExpr(v) * Heaviside(t))
         self.v0 = v
 
 
@@ -862,7 +904,7 @@ class Vdc(VoltageSource):
 
         self.args = (v, )
         v = cExpr(v)
-        self._Voc = Vsuper(Vconst(v, dc=True))
+        self._Voc = Voltage(Vconst(v, dc=True))
         self.v0 = v
 
     @property
@@ -902,7 +944,7 @@ class Vac(VoltageSource):
         self.omega = omega
         self.v0 = V
         self.phi = phi
-        self._Voc = Vsuper(Vphasor(self.v0 * exp(j * self.phi),
+        self._Voc = Voltage(Vphasor(self.v0 * exp(j * self.phi),
                                    ac=True, omega=self.omega))
 
     @property
@@ -918,9 +960,9 @@ class Vnoise(VoltageSource):
 
     def __init__(self, V, nid=None):
 
-        V1 = Vn(V, nid=nid)
+        V1 = Vnoisy(V, nid=nid)
         self.args = (V, V1.nid)
-        self._Voc = Vsuper(V1)
+        self._Voc = Voltage(V1)
 
         
 class v(VoltageSource):
@@ -930,7 +972,7 @@ class v(VoltageSource):
 
         self.args = (vval, )
         Vval = tExpr(vval)
-        self._Voc = Vsuper(Vval)
+        self._Voc = Voltage(Vval)
 
 
 class CurrentSource(OnePort):
@@ -955,7 +997,7 @@ class sI(CurrentSource):
 
         self.args = (Ival, )
         Ival = sExpr(Ival)
-        self._Isc = Isuper(Is(Ival))
+        self._Isc = Current(Is(Ival))
 
 
 class I(CurrentSource):
@@ -964,7 +1006,7 @@ class I(CurrentSource):
     def __init__(self, Ival):
 
         self.args = (Ival, )
-        self._Isc = Isuper(Ival)
+        self._Isc = Current(Ival)
 
             
 class Istep(CurrentSource):
@@ -976,7 +1018,7 @@ class Istep(CurrentSource):
 
         self.args = (i, )
         i = cExpr(i)
-        self._Isc = Isuper(tExpr(i) * Heaviside(t), causal=True)
+        self._Isc = Current(tExpr(i) * Heaviside(t))
         self.i0 = i
 
 
@@ -990,7 +1032,7 @@ class Idc(CurrentSource):
 
         self.args = (i, )
         i = cExpr(i)
-        self._Isc = Isuper(Iconst(i, dc=True))
+        self._Isc = Current(Iconst(i, dc=True))
         self.i0 = i
 
     @property
@@ -1028,7 +1070,7 @@ class Iac(CurrentSource):
         self.omega = omega
         self.i0 = I
         self.phi = phi
-        self._Isc = Isuper(Iphasor(self.i0 * exp(j * self.phi),
+        self._Isc = Current(Iphasor(self.i0 * exp(j * self.phi),
                                    ac=True, omega=self.omega))
 
     @property
@@ -1044,8 +1086,8 @@ class Inoise(CurrentSource):
 
     def __init__(self, I, nid=None):
 
-        I1 = In(I, nid=nid)
-        self._Isc = Isuper(I1)
+        I1 = Inoisy(I, nid=nid)
+        self._Isc = Current(I1)
         self.args = (I, I1.nid)
 
         
@@ -1056,7 +1098,7 @@ class i(CurrentSource):
 
         self.args = (ival, )
         Ival = tExpr(ival)
-        self._Isc = Isuper(Ival)
+        self._Isc = Current(Ival)
 
 
 class Xtal(OnePort):
@@ -1074,7 +1116,7 @@ class Xtal(OnePort):
         self.L1 = cExpr(L1)
         self.C1 = cExpr(C1)
 
-        self._Z = self.expand().Z
+        self._Z = self.expand().impedance
         self.args = (C0, R1, L1, C1)
 
     def expand(self):
@@ -1102,7 +1144,7 @@ class FerriteBead(OnePort):
         self.Cp = cExpr(Cp)
         self.Lp = cExpr(Lp)
 
-        self._Z = self.expand().Z
+        self._Z = self.expand().impedance
         self.args = (Rs, Rp, Cp, Lp)
 
     def expand(self):
@@ -1153,14 +1195,95 @@ class LoadCircuit(Network):
         # TODO: draw this better rather than as a oneport.
         return self.vnet.net_make(self.vnet, n1, n2)
 
+
+class ControlledSource(OnePort):
+    """These components are controlled one-ports."""
+    pass
+
+
+class CCVS(ControlledSource):
+
+    def __init__(self, control, value):
+
+        self.args = (control, value)        
+        self._Voc = Voltage(0)
+        self._Z = Impedance(0)
+
+        
+class CCCS(ControlledSource):
+
+    def __init__(self, control, value):
+
+        self.args = (control, value)        
+        self._Isc = Current(0)
+        self._Y = Admittance(0)     
+
+        
+class VCVS(ControlledSource):
+
+    def __init__(self, value):
+
+        self.args = (value, )        
+        self._Voc = Voltage(0)
+        self._Z = Impedance(0)
+
+        
+class VCCS(ControlledSource):
+
+    def __init__(self, value):
+
+        self.args = (value, )
+        self._Isc = Current(0)
+        self._Y = Admittance(0)     
+        
+
+class Dummy(OnePort):
+
+    def __init__(self, *args):
+
+        self.args = args
+
+
+class K(Dummy):
+    """Coupling coefficient"""
+
+    def __init__(self, L1, L2, K):
+
+        self.args = (L1, L2, K)
+        self.K = cExpr(K)
+
+
+class W(Dummy):
+    """Wire (short)"""
+
+    def __init__(self):
+
+        self.args = ()
+        self._Z = Impedance(0)
+
+        
+class O(Dummy):
+    """Open circuit"""
+
+    def __init__(self):
+
+        self.args = ()
+        self._Y = Admittance(0)
+
+        
+class P(O):
+    """Port (open circuit)"""
+    pass
+
     
 # Imports at end to circumvent circular dependencies
 from .expr import Expr
 from .cexpr import cExpr, Iconst, Vconst
 from .sexpr import sExpr, Is, Vs, Ys, Zs
 from .texpr import tExpr
-from .noiseexpr import In, Vn
-from .super import Isuper, Vsuper
+from .noiseexpr import Inoisy, Vnoisy
+from .voltage import Voltage
+from .current import Current
 from .phasor import Iphasor, Vphasor
 from .twoport import Ladder, LSection, TSection
 
