@@ -1,6 +1,7 @@
 import re
-from .config import print_expr_map, functions, words, subscripts, junicode
-from .latex import latex_str
+from .config import latex_expr_map, pretty_expr_map, str_expr_map
+from .config import functions, words, subscripts
+from .latex import latex_str, latex_double_sub
 from sympy.printing.str import StrPrinter
 from sympy.printing.latex import LatexPrinter
 from sympy.printing.pretty.pretty import PrettyPrinter
@@ -12,10 +13,13 @@ __all__ = ('pretty', 'pprint', 'latex', 'print_str')
 #
 # jupyter looks for methods called _repr_latex_ and _repr_pretty_.
 #
+# IPython looks for _repr_pretty
+#
 # sympy.latex() looks for methods called _latex
 
 # LaTeX markup is nicer but it requires mathjax.
 
+# FIXME: should import from parser
 cpt_names = ('C', 'E', 'F', 'G', 'H', 'I', 'L', 'R', 'V', 'Y', 'Z', 'i', 'v')
 cpt_name_pattern = re.compile(r"(%s)([\w']*)" % '|'.join(cpt_names))
 sub_super_pattern = re.compile(r"([_\^]){([\w]+)}")
@@ -40,6 +44,12 @@ def canonical_name(name):
     # Convert R_{out} to R_out for SymPy to recognise.
     name = sub_super_pattern.sub(foo, name)
 
+    # Convert v_Cfoo to v_C_foo, etc.   This is required for
+    # state-space state variables.
+    if (len(name) >= 3 and name[0:2] in ('v_', 'V_', 'i_', 'I_') and
+        name[2] in ('L', 'C')):
+        return name[0:2] + canonical_name(name[2:])
+    
     if name.find('_') != -1:
         return name
 
@@ -65,7 +75,7 @@ def canonical_name(name):
     # Convert irms to i_rms, etc.
     if name[1:].lower() in subscripts:
         return name[0] + '_' + name[1:]    
-    
+
     return name
 
 
@@ -73,14 +83,16 @@ class LcapyStrPrinter(StrPrinter):
 
     def _print(self, expr):
 
-        if expr is sym.I:
-            return 'j'
-        
         from .expr import Expr
         if isinstance(expr, Expr):
             expr = expr.expr
-            if expr in print_expr_map:
-                return print_expr_map[expr]        
+
+        # Convert sym.I to j etc.
+        try:            
+            if expr in pretty_expr_map:        
+                return str_expr_map[expr]
+        except:
+            pass
         return super(LcapyStrPrinter, self)._print(expr)
 
     def _print_Symbol(self, expr):
@@ -96,16 +108,52 @@ class LcapyLatexPrinter(LatexPrinter):
 
         from .expr import Expr
         if isinstance(expr, Expr):        
-            expr = expr.expr            
-            if expr in print_expr_map:
-                return print_expr_map[expr]
+            expr = expr.expr
+
+        # Convert sym.I to j etc.
+        try:
+            if expr in latex_expr_map:
+                return latex_expr_map[expr]
+        except:
+            pass
         return super(LcapyLatexPrinter, self)._print(expr)
 
+    def _print_Heaviside(self, expr, exp=None):
+
+        tex = r"%s\left(%s\right)" % (latex_expr_map[sym.Heaviside],
+                                      self._print(expr.args[0]))
+        if exp:
+            tex = r"\left(%s\right)^{%s}" % (tex, exp)
+        return tex
+
+    def _print_UnitImpulse(self, expr, exp=None):
+
+        tex = r"\delta\left[%s\right]" % self._print(expr.args[0])
+        if exp:
+            tex = r"\delta\left[%s\right]^{%s}" % (tex, exp)
+        return tex    
+    
     def _print_Symbol(self, expr):
+
+        # Note, SymPy prints atan2 as atan_{2}.
+
         expr = sym.Symbol(canonical_name(expr.name))                
-        return super(LcapyLatexPrinter, self)._print_Symbol(expr)    
+        parts = expr.name.split('_')        
+        s = super(LcapyLatexPrinter, self)._print_Symbol(expr)
+        if len(parts) >= 2:
+            # Sympy cannot print a symbol name with a double subscript
+            # using LaTeX.  This should be fixed in Sympy.
+            # Need to convert v_{C 1} to v_{C_{1}}
+            parts2 = s.split(' ')
+            if len(parts2) >= 2:
+                s = parts2[0] + '_{' + ''.join(parts2[1:])[:-1] + '}}'
+
+        #s = latex_double_sub(s)
+        s = latex_str(s)
+        return s
 
     def _print_AppliedUndef(self, expr):
+
         name = canonical_name(expr.func.__name__)
         args = [str(self._print(arg)) for arg in expr.args]        
         return '%s(%s)' % (name, ','.join(args))
@@ -115,21 +163,44 @@ class LcapyPrettyPrinter(PrettyPrinter):
 
     def _print(self, expr):
 
-        if expr is sym.I:
-            return self._print_basestring(junicode)
-        
         from .expr import Expr
         if isinstance(expr, Expr):        
             expr = expr.expr
-            if expr in print_expr_map:
-                return self._print_basestring(print_expr_map[expr])
+
+        try:            
+            if expr in pretty_expr_map:
+                return self._print_basestring(pretty_expr_map[expr])
+        except:
+            pass
         return super(LcapyPrettyPrinter, self)._print(expr)
 
     def _print_Symbol(self, expr):
+
         expr = sym.Symbol(canonical_name(expr.name))                
-        return super(LcapyPrettyPrinter, self)._print_Symbol(expr)
+        parts = expr.name.split('_')        
+        if len(parts) >= 2:
+            # Due to unicode limitations, Sympy cannot print a symbol
+            # name with a double subscript.  As a work-around combine
+            # the subscripts.  Note, Sympy converts 'v_C1' into
+            # 'v_C_1' so we need to clean up.
+            expr.name = parts[0] + '_' + ''.join(parts[1:])
+        s = super(LcapyPrettyPrinter, self)._print_Symbol(expr)
+        return s
 
+    def _print_UnitImpulse(self, e):
 
+        from sympy.printing.pretty.stringpict import prettyForm
+        from sympy.printing.pretty.pretty_symbology import greek_unicode
+        
+        if self._use_unicode:
+            pform = self._print(e.args[0])
+            pform = prettyForm(*pform.parens(left='[', right=']'))
+            pform = prettyForm(*pform.left(greek_unicode['delta']))
+            return pform
+        else:
+            return self._print_Function(e)
+    
+    
 def print_str(expr):
     """Convert expression into a string."""
     
@@ -138,7 +209,7 @@ def print_str(expr):
 
 def pretty(expr, **settings):
     """Pretty print an expression."""
-    
+
     return LcapyPrettyPrinter(settings).doprint(expr)
 
 
@@ -192,10 +263,12 @@ def latex(expr, fold_frac_powers=False, fold_func_brackets=False,
         # operatorname requires amsmath so switch to mathrm
         string = r'\mathrm{%s}' % match.groups()[0].replace('\\_', '_')
 
-    return latex_str(string)
+    return string
 
 
+# See sympy/interactive/printing.py and IPython/core/formatters.py
+# Also see hack at end of expr.py to support latex for Lcapy container
+# types.
+
 from sympy import init_printing
 init_printing(latex_printer=latex, pretty_printer=pretty, str_printer=print_str)
-
-        
