@@ -1,3 +1,9 @@
+"""This module provides miscellaneous support for schematic drawing.
+
+Copyright 2014--2020 Michael Hayes, UCECE
+
+"""
+
 import numpy as np
 
 class Pos(object):
@@ -23,6 +29,13 @@ class Pos(object):
 
         return Pos(self.x + arg.x, self.y + arg.y)
 
+    def __sub__(self, arg):
+
+        if not isinstance(arg, Pos):
+            arg = Pos(arg)
+
+        return Pos(self.x - arg.x, self.y - arg.y)    
+
     def __str__(self):
 
         xstr = ('%.3f' % self.x).rstrip('0').rstrip('.')
@@ -77,9 +90,6 @@ class Opts(dict):
 
         if string == '':
             return
-        if string[0] == ';':
-            self['append'] += string[1:] + '\n'
-            return
 
         for part in split(string):
             part = part.strip()
@@ -117,6 +127,9 @@ class Opts(dict):
         
         return self.__class__(super(Opts, self).copy())
 
+    def remove(self, arg):
+        self.pop(arg)
+            
     def strip(self, *args):
 
         stripped = Opts()
@@ -143,3 +156,68 @@ class Opts(dict):
         self.strip_voltage_labels()
         self.strip_current_labels()
         self.strip_labels()
+
+        
+class Steps(object):
+
+    def __init__(self, string, pos1, pos2):
+
+        self.string = string
+
+        self.numv = 0
+        self.numh = 0
+        self.steps = []
+        self.pos1 = pos1
+        self.pos2 = pos2
+        self.pos = pos1
+
+        self.dx = (pos2.x - pos1.x)
+        self.dy = (pos2.y - pos1.y)
+        
+        if string == '':
+            if abs(self.dx) > abs(self.dy):
+                string = '-|'
+            else:
+                string = '|-'                
+                
+        s = string
+        m = 0
+        while m < len(s):
+            if s[m].isdigit():
+                n = m
+                while s[n].isdigit():
+                    n += 1
+                num = int(s[m:n])
+                m = n
+            else:
+                num = 1    
+            if s[m] == '|':
+                self.numv += num
+                self.steps.append((s[m], num))
+            elif s[m] == '-':
+                self.numh += num
+                self.steps.append((s[m], num))                
+            else:
+                raise ValueError('Expecting - or | in %s' % string)
+            m += 1
+
+        self.dh = self.dx / self.numh
+        self.dv = self.dy / self.numv
+            
+    def __iter__(self):
+        self.n = 0
+        self.pos = self.pos1
+        return self
+
+    def __next__(self):
+        if self.n >= len(self.steps):
+            raise StopIteration
+
+        c, n = self.steps[self.n]
+        if c == '-':
+            self.pos += Pos(self.dh * n, 0)
+        elif c == '|':
+            self.pos += Pos(0, self.dv * n)
+
+        self.n += 1
+        return self.pos
