@@ -1,14 +1,25 @@
+"""This file provides the sExpr class to represent s-domain (Laplace
+domain) expressions.
+
+Copyright 2014--2019 Michael Hayes, UCECE
+
+"""
+
 from __future__ import division
 from .laplace import inverse_laplace_transform
-from .sfwexpr import sfwExpr
 from .sym import ssym, tsym, j, pi
 from .vector import Vector
 from .ratfun import _zp2tf, Ratfun
+from .expr import Expr, symbol, expr, ExprDict
+from .functions import sqrt
 import sympy as sym
 import numpy as np
 
 
-class sExpr(sfwExpr):
+__all__ = ('Hs', 'Is', 'Vs', 'Ys', 'Zs', 'zp2tf', 'tf')
+
+
+class sExpr(Expr):
     """s-domain expression or symbol."""
 
     var = ssym
@@ -22,15 +33,15 @@ class sExpr(sfwExpr):
             raise ValueError(
                 's-domain expression %s cannot depend on t' % self.expr)
 
-    def differentiate(self):
-        """Differentiate (multiply by s)."""
+    def tdifferentiate(self):
+        """Differentiate in t-domain (multiply by s)."""
 
-        return self.__class__(self.expr * self.var)
+        return self.__class__(self.expr * self.var, **self.assumptions)
 
-    def integrate(self):
-        """Integrate (divide by s)."""
+    def tintegrate(self):
+        """Integrate in t-domain (divide by s)."""
 
-        return self.__class__(self.expr / self.var)
+        return self.__class__(self.expr / self.var, **self.assumptions)
 
     def delay(self, T):
         """Apply delay of T seconds by multiplying by exp(-s T)."""
@@ -93,7 +104,7 @@ class sExpr(sfwExpr):
         """Convert to Fourier domain."""
         from .symbols import f
         
-        if assumptions.get('causal', False) or self.is_causal:
+        if assumptions.get('causal', self.is_causal):
             return self.subs(j * 2 * pi * f)
 
         return self.time(**assumptions).fourier(**assumptions)
@@ -160,7 +171,7 @@ class sExpr(sfwExpr):
             raise (ValueError, 't values not equally spaced')
 
         # Perform polynomial long division so expr = Q + M / D                
-        N, D, delay = self.decompose()
+        N, D, delay = self._decompose()
         Q, M = sym.div(N, D)
         expr = M / D
 
@@ -194,7 +205,7 @@ class sExpr(sfwExpr):
 
         return y
 
-    def decompose(self):
+    def _decompose(self):
 
         N, D, delay = Ratfun(self, s).as_ratfun_delay()                
 
@@ -210,25 +221,84 @@ class sExpr(sfwExpr):
         from .plot import plot_pole_zero
         return plot_pole_zero(self, **kwargs)
 
-    def coeffs(self):
-        """Return list of coeffs assuming the expr is a polynomial in s.  The
-        highest powers come first.  This will fail for a rational
-        function.
+    def parameterize(self, zeta=True):
+        """Parameterize first and second-order expressions.
+
+        For first order systems, parameterize as:
+
+        K * (s + beta) / (s + alpha)
+
+        K / (s + alpha)
+
+        K (s + beta)
+
+        where appropriate.
+
+        If `zeta` is True, parameterize second-order expression in
+        standard form using damping factor and natural frequency
+        representation, i.e.
+
+        N(s) / (s**2 + 2 * zeta * omega_0 * s + omega_0**2)
+        
+        otherwise parameterize as
+        
+        N(s) / (s**2 + 2 * sigma_1 * s + omega_1**2 + sigma_1**2)
 
         """
-        z = sym.Poly(self.expr, self.var)
-        return z.all_coeffs()
 
-    def normcoeffs(self):
-        """Return list of coeffs (normalised so the highest power is 1)
-        assuming the expr is a polynomial in s.  The highest powers
-        come first.  This will fail for a rational function.
+        def def1(defs, symbolname, value):
+            sym1 = symbol(symbolname)
+            defs[symbolname] = value
+            return sym1
 
-        """
-        z = sym.Poly(self.expr, self.var)
-        c = z.all_coeffs()
-        return [sym.simplify(c1 / c[0]) for c1 in c]
+        ndegree = self.N.degree        
+        ddegree = self.D.degree
+        ncoeffs = self.N.coeffs(norm=True)
+        dcoeffs = self.D.coeffs(norm=True)
 
+        defs = ExprDict()
+
+        K = self.K
+        if ndegree < 1 and ddegree < 1:
+            return self, defs
+        if ndegree == 1 and ddegree == 1:
+            K = def1(defs, 'K', K)
+            alpha = def1(defs, 'alpha', dcoeffs[1])
+            beta = def1(defs, 'beta', ncoeffs[1])
+            return K * (s + beta) / (s + alpha), defs
+        if ndegree == 1 and ddegree == 0:
+            K = def1(defs, 'K', K)
+            beta = def1(defs, 'beta', ncoeffs[1])
+            return K * (s + beta), defs
+        if ndegree == 0 and ddegree == 1:
+            K = def1(defs, 'K', K)
+            alpha = def1(defs, 'alpha', dcoeffs[1])
+            return K / (s + alpha), defs
+        if ddegree == 2:
+            K = def1(defs, 'K', K)
+            coeffs = self.N.coeffs()
+
+            if not zeta:
+                sigma1 = def1(defs, 'sigma_1', dcoeffs[1] / 2)
+                omega1 = def1(defs, 'omega_1',
+                              sqrt(dcoeffs[2] - (dcoeffs[1] / 2)**2).simplify())
+                return K * (self.N / coeffs[0]) / (s**2 + 2 * sigma1 * s + sigma1**2 + omega1**2), defs
+                
+            omega0 = def1(defs, 'omega_0', sqrt(dcoeffs[2]))
+            zeta = def1(defs, 'zeta', dcoeffs[1] / (2 * sqrt(dcoeffs[2])))
+            return K * (self.N / coeffs[0]) / (s**2 + 2 * zeta * omega0 * s + omega0**2), defs
+        
+        return self, defs
+
+    def bilinear_transform(self):
+
+        from .discretetime import z, dt
+
+        # s = ln(z) / dt gives the exact solution
+        
+        return self.subs((2 / dt) * (1 - z**-1) / (1 + z**-1))
+        
+    
     
 # Perhaps use a factory to create the following classes?
 
@@ -239,31 +309,11 @@ class Zs(sExpr):
     quantity = 'Impedance'
     units = 'ohms'
 
-    def __init__(self, val, **assumptions):
+    def __init__(self, val, causal=True, **assumptions):
 
-        super(Zs, self).__init__(val, **assumptions)
+        super(Zs, self).__init__(val, causal=causal, **assumptions)
         self._laplace_conjugate_class = Zt
 
-    @classmethod
-    def C(cls, Cval):
-
-        return cls(1 / (s * Cval), causal=True)
-
-    @classmethod
-    def G(cls, Gval):
-
-        return cls(1 / Gval, causal=True)
-
-    @classmethod
-    def L(cls, Lval):
-
-        return cls(s * Lval, causal=True)
-
-    @classmethod
-    def R(cls, Rval):
-
-        return cls(Rval, causal=True)
-
     def cpt(self):
         from .oneport import R, C, L, Z
 
@@ -290,31 +340,11 @@ class Ys(sExpr):
     quantity = 'Admittance'
     units = 'siemens'
 
-    def __init__(self, val, **assumptions):
+    def __init__(self, val, causal=True, **assumptions):
 
-        super(Ys, self).__init__(val, **assumptions)
+        super(Ys, self).__init__(val, causal=causal, **assumptions)
         self._laplace_conjugate_class = Yt
 
-    @classmethod
-    def C(cls, Cval):
-
-        return cls(s * Cval, causal=True)
-
-    @classmethod
-    def G(cls, Gval):
-
-        return cls(Gval, causal=True)
-
-    @classmethod
-    def L(cls, Lval):
-
-        return cls(1 / (s * Lval), causal=True)
-
-    @classmethod
-    def R(cls, Rval):
-
-        return cls(1 / Rval, causal=True)
-
     def cpt(self):
         from .oneport import G, C, L, Y
 
@@ -424,6 +454,14 @@ def zp2tf(zeros, poles, K=1, var=None):
     return Hs(_zp2tf(zeros, poles, K, var))
 
 
+def sexpr(arg):
+    """Create sExpr object.  If `arg` is ssym return s"""
+
+    if arg is ssym:
+        return s
+    return sExpr(arg)
+
+
 from .texpr import Ht, It, Vt, Yt, Zt, tExpr
 s = sExpr('s')
 
