@@ -5,7 +5,7 @@ from .acdc import is_ac
 from .printing import pprint, pretty, latex
 import six
 
-__all__ = ('Super', 'Vsuper', 'Isuper')
+__all__ = ('Super', 'Voltage', 'Current')
 
 class Super(ExprDict):
     """This class represents a superposition of different signal types:
@@ -27,7 +27,7 @@ class Super(ExprDict):
     each of the noise components in quadrature since they are
     independent.
 
-    For example, consider V = Vsuper('cos(3 * t) + exp(-4 * t) + 5')
+    For example, consider V = Voltage('cos(3 * t) + exp(-4 * t) + 5')
 
     str(V(t)) gives 'cos(3*t) + 5 + exp(-4*t)'
 
@@ -65,9 +65,9 @@ class Super(ExprDict):
         # hides the underlying way the signal is analysed.
         return self.decompose()
 
-    def latex(self):
+    def latex(self, **kwargs):
         """Latex"""
-        return latex(self._representation())
+        return latex(self._representation(), **kwargs)
     
     def _repr_pretty_(self, p, cycle):
         """This is used by jupyter notebooks to display an expression using
@@ -158,7 +158,7 @@ class Super(ExprDict):
 
     @property
     def has_t(self):
-        """True if have transient component defined in the time-domain."""        
+        """True if have transient component defined in the time-domain."""
         return 't' in self
 
     @property
@@ -204,7 +204,7 @@ class Super(ExprDict):
     
     @property
     def is_causal(self):
-        return self.is_transient and self.s.is_causal
+        return (self.is_transient and self.s.is_causal) or self == 0
 
     @property
     def is_superposition(self):
@@ -281,41 +281,26 @@ class Super(ExprDict):
         return new
 
     def __eq__(self, x):
+        """ Test for mathematical equality as far as possible.
+        This cannot be guaranteed since it depends on simplification.
+        Note, SymPy comparison is for structural equality.  
 
-        # Cannot compare noise by subtraction.
-        if isinstance(x, Super):
-            # TODO, be smarter about transformations
-            x = x.decompose()
-            y = self.decompose()            
-            if y.items() != x.items():
-                return False
-            for kind, value in y.items():
-                if value != x[kind]:
-                    return False
-            return True
+        Also note, noise cannot be compared."""
         
-        diff = self - x
+        diff = (self - x).simplify().decompose()
         for kind, value in diff.items():
             if value != 0:
                 return False
         return True
 
-
     def __ne__(self, x):
+        """ Test for mathematical inequality as far as possible.
+        This cannot be guaranteed since it depends on simplification.
+        Note, SymPy comparison is for structural equality.  
 
-        # Cannot compare noise by subtraction.
-        if isinstance(x, Super):
-            # TODO, be smarter about transformations
-            x = x.decompose()
-            y = self.decompose()            
-            if y.items() != x.items():
-                return True
-            for kind, value in y.items():
-                if value != x[kind]:
-                    return True
-            return False
+        Also note, noise cannot be compared."""
         
-        diff = self - x
+        diff = (self - x).simplify().decompose()
         for kind, value in diff.items():
             if value != 0:
                 return True
@@ -547,6 +532,11 @@ class Super(ExprDict):
             self = self.decompose()        
         return ExprDict({k: v for k, v in self.items() if k in self.ac_keys()})
 
+    @property
+    def transient(self):
+        """Return the transient component."""        
+        return self.select('s').time()
+    
     @property
     def s(self):
         """Return the s-domain representation of the transient component.
@@ -637,193 +627,15 @@ class Super(ExprDict):
 
         return new
 
-
-class Vsuper(Super):
-
-    def __init__(self, *args, **kwargs):
-        self.type_map = {cExpr: Vconst, sExpr : Vs, noiseExpr: Vn,
-                         omegaExpr: Vphasor, tExpr : Vt}
-        self.decompose_domains = {'s': Vs, 'ac': Vphasor, 'dc':
-                                  Vconst, 'n': Vn, 't': Vt}
-        self.time_class = Vt
-        self.laplace_class = Vs    
-
-        super (Vsuper, self).__init__(*args, **kwargs)
-        
-    def __rmul__(self, x):
-        return self.__mul__(x)
-
-    def __mul__(self, x):
-        if isinstance(x, (int, float)):
-            return self.__scale__(x)
-
-        if isinstance(x, Super):
-            raise TypeError('Cannot multiply types %s and %s. '
-            'You need to extract a specific component, e.g., a.s * b.s' %
-            (type(self).__name__, type(x).__name__))
-
-        if not isinstance(x, Ys):
-            raise TypeError("Unsupported types for *: 'Vsuper' and '%s'" %
-                            type(x).__name__)
-        obj = self
-        if x.has(s):
-            obj = self.decompose()
-        
-        new = Isuper()
-        if 'dc' in obj:
-            # TODO, fix types
-            new += Iconst(obj['dc'] * cExpr(x.jomega(0)))
-        for key in obj.ac_keys():
-            new += obj[key] * x.jomega(obj[key].omega)
-        for key in obj.noise_keys():            
-            new += obj[key] * x.jomega
-        if 's' in obj:
-            new += obj['s'] * x
-        if 't' in obj:
-            new += self['t'] * tExpr(x)
-            
-        return new
-
-    def __div__(self, x):
-        if isinstance(x, (int, float)):
-            return self.__scale__(1 / x)
-
-        if isinstance(x, Super):
-            raise TypeError("""
-Cannot divide types %s and %s.  You need to extract a specific component, e.g., a.s / b.s.  If you want a transfer function use a.laplace() / b.laplace()""" % (type(self).__name__, type(x).__name__))
-
-        if not isinstance(x, Zs):
-            raise TypeError("Unsupported types for /: 'Vsuper' and '%s'" %
-                            type(x).__name__)
-        return self * Ys(1 / x)
-
-    def __truediv__(self, x):
-        return self.__div__(x)
-
-    def cpt(self):
-        from .oneport import V
-        # Perhaps should generate more specific components such as Vdc?
-        return V(self.time())
-
-class Isuper(Super):
-
-    def __init__(self, *args, **kwargs):    
-
-        self.type_map = {cExpr: Iconst, sExpr : Is, noiseExpr: In,
-                         omegaExpr: Iphasor, tExpr : It}
-        self.decompose_domains = {'s': Is, 'ac': Iphasor, 'dc':
-                                  Iconst, 'n': In, 't': It}
-        self.time_class = It
-        self.laplace_class = Is
-
-        super (Isuper, self).__init__(*args, **kwargs)
-
-    def __rmul__(self, x):
-        return self.__mul__(x)
+    def oneport(self):
+        """Create oneport component."""
+        return self.cpt()
     
-    def __mul__(self, x):
-        if isinstance(x, (int, float)):
-            return self.__scale__(x)
-
-        if isinstance(x, Super):
-            raise TypeError('Cannot multiply types %s and %s. '
-            'You need to extract a specific component, e.g., a.s * b.s' %
-            (type(self).__name__, type(x).__name__))
-        
-        if not isinstance(x, Zs):
-            raise TypeError("Unsupported types for *: 'Isuper' and '%s'" %
-                            type(x).__name__)
-        obj = self
-        if x.has(s):
-            obj = self.decompose()
-
-        new = Vsuper()
-        if 'dc' in obj:
-            # TODO, fix types
-            new += Vconst(obj['dc'] * cExpr(x.jomega(0)))
-        for key in obj.ac_keys():
-            new += obj[key] * x.jomega(obj[key].omega)
-        for key in obj.noise_keys():            
-            new += obj[key] * x.jomega            
-        if 's' in obj:
-            new += obj['s'] * x
-        if 't' in obj:
-            new += obj['t'] * tExpr(x)                        
-        return new
-
-    def __div__(self, x):
-        if isinstance(x, (int, float)):
-            return self.__scale__(1 / x)
-
-        if isinstance(x, Super):
-            raise TypeError('Cannot divide types %s and %s. '
-            'You need to extract a specific component, e.g., a.s / b.s' %
-            (type(self).__name__, type(x).__name__))
-
-        if not isinstance(x, Ys):
-            raise TypeError("Unsupported types for /: 'Isuper' and '%s'" %
-                            type(x).__name__)
-        return self * Zs(1 / x)
-
-    def __truediv__(self, x):
-        return self.__div__(x)
-
-    def cpt(self):
-        from .oneport import I
-        # Perhaps should generate more specific components such as Idc?        
-        return I(self.time())
-
-
-def Vname(name, kind, cache=False):
-    
-    if kind == 's':
-        return Vs(name + '(s)')
-    elif kind == 't':
-        return Vt(name + '(t)')
-    elif kind in (omegasym, omega, 'ac'):
-        return Vphasor(name + '(omega)')
-    # Not caching is a hack to avoid conflicts of Vn1 with Vn1(s) etc.
-    # when using subnetlists.  The alternative is a proper context
-    # switch.  This would require every method to set the context.
-    return expr(name, cache=cache)            
-
-
-def Iname(name, kind, cache=False):
-    
-    if kind == 's':
-        return Is(name + '(s)')
-    elif kind == 't':
-        return It(name + '(t)')
-    elif kind in (omegasym, omega, 'ac'):    
-        return Iphasor(name + '(omega)')
-    return expr(name, cache=cache)            
-
-
-def Vtype(kind):
-    
-    if isinstance(kind, str) and kind[0] == 'n':
-        return Vn
-    try:
-        return {'ivp' : Vs, 's' : Vs, 'n' : Vn,
-                'ac' : Vphasor, 'dc' : Vconst, 't' : Vt, 'time' : Vt}[kind]
-    except KeyError:
-        return Vphasor
-
-
-def Itype(kind):
-    if isinstance(kind, str) and kind[0] == 'n':
-        return In
-    try:
-        return {'ivp' : Is, 's' : Is, 'n' : In,
-                'ac' : Iphasor, 'dc' : Iconst, 't' : It, 'time' : It}[kind]
-    except KeyError:
-        return Iphasor
-    
-from .cexpr import Iconst, Vconst, cExpr        
+from .cexpr import cExpr        
 from .fexpr import fExpr    
-from .sexpr import Is, Vs, Ys, Zs, sExpr
-from .texpr import It, Vt, tExpr
-from .noiseexpr import In, Vn, noiseExpr
-from .phasor import Iphasor, Vphasor, Phasor
+from .sexpr import sExpr
+from .texpr import tExpr
+from .noiseexpr import noiseExpr
+from .phasor import Phasor
 from .omegaexpr import omegaExpr
 from .symbols import s, omega
