@@ -1,4 +1,12 @@
-from .config import exclude, aliases
+"""This file contains functions for handling symbols and in particular
+for integrating with SymPy.
+
+Copyright 2014--2020 Michael Hayes, UCECE
+
+"""
+
+
+from .config import excludes, aliases
 from sympy.parsing.sympy_parser import parse_expr, auto_number, rationalize
 try:
     from sympy.parsing.sympy_parser import NUMBER, NAME, OP        
@@ -9,7 +17,7 @@ from sympy import Basic, Symbol, Expr, Atom
 from sympy.core.function import AppliedUndef
 import sympy as sym
 import re
-from .context import context
+from .state import state
 
 __all__ = ('symsymbol', 'sympify', 'simplify')
 
@@ -20,7 +28,9 @@ exec('from sympy import *', global_dict)
 for _alias, _name in aliases.items():
     global_dict[_alias] = global_dict[_name]
 
-for _symbol in exclude:
+global_dict['abs'] = sym.Abs
+
+for _symbol in excludes:
     global_dict.pop(_symbol)
 
     
@@ -30,6 +40,7 @@ def capitalize_name(name):
 
 
 def symbol_name(symbol):
+
     return str(symbol)
 
 
@@ -126,6 +137,8 @@ def parse(string, symbols={}, evaluate=True, local_dict={}, **assumptions):
         if name not in symbols:
             if name == 'ivp':
                 raise ValueError('Huh')
+            if False:
+                print('Adding symbol %s' % name)
             symbols[name] = symbol
 
     return s
@@ -173,14 +186,18 @@ def sympify1(arg, symbols={}, evaluate=True, **assumptions):
 def sympify(expr, evaluate=True, **assumptions):
     """Create a SymPy expression.
 
-    By default, symbols are assumed to be positive unless real is
-    defined.
+    By default, symbols are assumed to be positive if no assumptions
+    are explicitly defined.
+
+    Note, this will not modify previously defined symbols with the
+    same name.  Thus you cannot change the assumptions.
 
     """
-
-    if 'real' not in assumptions and 'positive' not in assumptions:
+    if assumptions == {}:
         assumptions['positive'] = True
-    return sympify1(expr, context.symbols, evaluate, **assumptions)
+        # Note this implies that imag is False.   Also note that all
+        # reals are considered complex (but with a zero imag part).
+    return sympify1(expr, state.context.symbols, evaluate, **assumptions)
 
 
 def symsymbol(name, **assumptions):
@@ -263,6 +280,25 @@ def symdebug(expr, s='', indent=0):
     return s
 
 
+def symbol_map(name):
+
+    new = name
+    if not isinstance(name, str):
+        name = str(name)
+    
+    # Replace symbol names with symbol definitions to
+    # avoid problems with real or positive attributes.
+    if name in state.context.symbols:
+        new = state.context.symbols[name]
+    elif name in state.global_context.symbols:
+        new = state.global_context.symbols[name]            
+    else:
+        # Perhaps have symbol defined using sympy?
+        pass
+    return new
+
+
+# The following are all SymPy symbols.
 ssym = symsymbol('s', real=False)
 tsym = symsymbol('t', real=True)
 fsym = symsymbol('f', real=True)
