@@ -1,12 +1,13 @@
-from os import system, path, remove, chdir, getcwd
+from os import system, path, remove, chdir, getcwd, stat
+from subprocess import call, DEVNULL
 import re
-from sys import platform
+import platform
 
 # System dependent functions
 
-# Note, sometime in 2018 ImageMagicks convert program by default
-# disallowed pdf file conversions.  A workaround is to edit
-# /etc/ImageMagick-6/policy.xml 
+# Note, sometime in 2018 ImageMagick's convert program by default
+# disallowed pdf file conversions.  A work-around is to edit
+# /etc/ImageMagick-6/policy.xml to enable this conversion.
 
 def tmpfilename(suffix=''):
 
@@ -28,36 +29,57 @@ def convert_pdf_svg(pdf_filename, svg_filename):
                            svg_filename)
 
 
-def convert_pdf_png_convert(pdf_filename, png_filename, oversample=1):
+def convert_pdf_png_convert(pdf_filename, png_filename, dpi=300):
 
-    if 'win' in platform:
-        # Windows has a program called convert, try magick convert
-        # for image magick convert.
-        system('magick convert -density %d %s %s' %
-               (oversample * 100, pdf_filename, png_filename))
-    else:
-        system('convert -density %d %s %s' %
-               (oversample * 100, pdf_filename, png_filename))
+    program = 'convert'    
+    if platform.system() == 'Windows':
+        program = 'magick convert'
+
+    command = '%s -density %d %s %s' % (program, int(dpi), pdf_filename,
+                                        png_filename)
+    system(command)
         
     if not path.exists(png_filename):
         raise RuntimeError('Could not generate %s with convert' % 
                            png_filename)
+    if stat(png_filename).st_size == 0:
+        raise RuntimeError('Could not generate %s with convert, empty file' % 
+                           png_filename)
+    
 
-def convert_pdf_png_pdftoppm(pdf_filename, png_filename, oversample=1):
+def convert_pdf_png_ghostscript(pdf_filename, png_filename, dpi=300):
+
+    program = 'gs'    
+    if platform.system() == 'Windows':
+        program = 'gswin32'
+        if platform.machine().endswith('64'):
+            program = 'gswin64'            
+            
+    command = '%s -q -dQUIET -dSAFER -dBATCH -dNOPAUSE -dNOPROMPT -dMaxBitmap=500000000 -dAlignToPixels=0 -dGridFitTT=2 -sDEVICE=pngalpha -dTextAlphaBits=4 -dGraphicsAlphaBits=4 -r%dx%d -sOutputFile=%s %s' % (program, int(dpi), int(dpi), png_filename, pdf_filename)
+
+    system(command)
+
+    
+def convert_pdf_png_pdftoppm(pdf_filename, png_filename, dpi=300):
 
     system('pdftoppm -r %d  -png %s -thinlinemode shape > %s' %
-           (oversample * 150, pdf_filename, png_filename))
+           (int(dpi), pdf_filename, png_filename))
         
     if not path.exists(png_filename):
         raise RuntimeError('Could not generate %s with pdftoppm' % 
                            png_filename)    
 
-def convert_pdf_png(pdf_filename, png_filename, oversample=1):
     
+def convert_pdf_png(pdf_filename, png_filename, dpi=300):
+
     try:
-        convert_pdf_png_pdftoppm(pdf_filename, png_filename, oversample)
+        convert_pdf_png_ghostscript(pdf_filename, png_filename, dpi)
     except:
-        convert_pdf_png_convert(pdf_filename, png_filename, oversample)        
+        try:
+            convert_pdf_png_convert(pdf_filename, png_filename, dpi)
+        except:
+            convert_pdf_png_pdftoppm(pdf_filename, png_filename, dpi)
+
 
 def latex_cleanup(tex_filename, wanted_filename=''):
 
@@ -82,7 +104,8 @@ def run_latex(tex_filename):
     if dirname != '':
         chdir(path.abspath(dirname))
         
-    system('pdflatex -interaction batchmode %s.tex' % baseroot)
+    call(['pdflatex', '-interaction', 'batchmode', '%s.tex' %
+          baseroot], stderr=DEVNULL, stdout=DEVNULL)
     
     if dirname != '':
         chdir(cwd)            
@@ -111,11 +134,14 @@ def circuitikz_version():
 
     log_filename = tex_filename.replace('.tex', '.log')
     run_latex(tex_filename)
-    
-    log = open(log_filename, 'r').read()
-    latex_cleanup(tex_filename)    
 
-    match = re.search(r'circuitikz ([0-9/]+)', log)
-    if match is None:
-        return None
-    return match.group(1)
+    with open(log_filename, 'rt') as logfile:
+        lines = logfile.readlines()
+        for m, line in enumerate(lines):
+            if line.startswith('Package: circuitikz'):
+                match = re.search(r'circuitikz ([0-9/]+)', line)
+                date = match.group(1)
+                version = lines[m + 1].strip()
+                return date, version
+    return None, None
+
