@@ -49,6 +49,14 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual2(a.sign, Expr(j), "sign incorrect.")
         self.assertEqual2(-a.sign, Expr(-j), "sign incorrect.")        
 
+
+    def test_cExpr1(self):
+        """Lcapy: check cExpr1
+
+        """
+        a = cExpr('1')
+        self.assertEqual2(a.evaluate(), 1, "evaluate incorrect.")
+        
         
     def test_sExpr1(self):
         """Lcapy: check sExpr1
@@ -57,6 +65,10 @@ class LcapyTester(unittest.TestCase):
         a = sExpr('(s+2)/(s-2)')
         self.assertEqual2(a.N, sExpr('s+2'), "N incorrect.")
         self.assertEqual2(a.D, sExpr('s-2'), "D incorrect.")
+        self.assertEqual2(a.Ndegree, 1, "N degree incorrect.")
+        self.assertEqual2(a.Ddegree, 1, "D degree incorrect.")
+        self.assertEqual2(a.degree, 1, "Degree incorrect.")
+        self.assertEqual2(a.strictly_proper, False, "strictly proper incorrect.")        
 
         self.assertEqual2(sorted(a.poles()), [2], "poles incorrect.")
         self.assertEqual2(sorted(a.zeros()), [-2], "zeros incorrect.")
@@ -102,6 +114,10 @@ class LcapyTester(unittest.TestCase):
         a = (s**2 + 5 * s + 6) / (s - 2)
         self.assertEqual2(a.N, s ** 2 + 5 * s + 6, "N incorrect.")
         self.assertEqual2(a.D, s - 2, "D incorrect.")
+        self.assertEqual2(a.Ndegree, 2, "N degree incorrect.")
+        self.assertEqual2(a.Ddegree, 1, "D degree incorrect.")
+        self.assertEqual2(a.degree, 2, "Degree incorrect.")
+        self.assertEqual2(a.strictly_proper, False, "strictly proper incorrect.")
 
         self.assertEqual2(sorted(a.poles()), [2], "poles incorrect.")
         self.assertEqual2(sorted(a.zeros()), sorted([-3, -2]), "zeros incorrect.")
@@ -192,12 +208,23 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(a.inverse_laplace(causal=True), t * exp(-4 * t) * Heaviside(t), "inverse Laplace incorrect.")        
 
 
-    def test_sExpr1(self):
+    def test_sExpr7(self):
         """Lcapy: check sExpr7 (delay)
 
         """
         a = sExpr('(s+1)*exp(-3*s)/((s+3)*(s+4))')
         self.assertEqual(a.inverse_laplace(causal=True), (3 * exp(-4 * t  + 12) - 2 * exp(-3 * t + 9)) * H(t- 3), "inverse Laplace incorrect.")
+
+
+    def test_sExpr8(self):
+        """Lcapy: check sExpr8 (jomega)
+
+        """
+        a = expr('a')
+        H = a * s / (a * s + 1)
+        Hw = H(j * omega)
+
+        self.assertEqual(Hw.phase, atan2(1, a * omega), 'phase')
         
 
     def test_wExpr1(self):
@@ -380,7 +407,7 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(Heaviside(-2 * t - 1).is_causal, False, "Heaviside(-2 * t - 1).is_causal")
         
     def test_noise(self):
-        a = Vn(2)
+        a = Vnoisy(2)
         self.assertEqual(a.nid, a.conjugate.nid, "Different nids for conjugate")
         self.assertEqual(a.nid, a.real.nid, "Different nids for real")
         self.assertEqual(a.nid, a.imag.nid, "Different nids for imag")
@@ -418,6 +445,14 @@ class LcapyTester(unittest.TestCase):
         
         self.assertEqual(isinstance(symbol('c'), Expr), True, "symbol")
 
+        if expr('f') is not f:
+            raise AssertionError('Not f')
+        if expr('s') is not s:
+            raise AssertionError('Not s')
+        if expr('t') is not t:
+            raise AssertionError('Not t')
+        if expr('omega') is not omega:
+            raise AssertionError('Not omega')
 
     def test_comparison(self):
 
@@ -455,3 +490,112 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(expr('4').limit(t, 0), 4, "limit")
         self.assertEqual(expr('t + 4').limit(t, 0), 4, "limit")        
         
+    def test_parameterize(self):
+
+        a = 2 / (s + 3)
+        p, defs = a.parameterize()
+        self.assertEqual(a, p.subs(defs), "parameterize")
+
+        a =  (s + 3) / 6
+        p, defs = a.parameterize()
+        self.assertEqual(a, p.subs(defs), "parameterize")
+
+        a =  (2 * s + 3) / (s + 4)
+        p, defs = a.parameterize()
+        self.assertEqual(a, p.subs(defs), "parameterize")
+
+        a =  (s + 3) / (s**2 + 4 * s + 4)
+        p, defs = a.parameterize()
+        self.assertEqual(a, p.subs(defs), "parameterize")
+
+    def test_integrate(self):
+
+        self.assertEqual(t.integrate((t, 0, t)), t**2 / 2, "integrate t")
+        
+
+    def test_partfrac(self):
+
+        H = expr('F(s)') / (s**2 + 3 * s + 6)
+        
+        self.assertEqual(H.partfrac(), H,  "undef partfrac")
+        self.assertEqual(H.partfrac(True), H,  "undef partfrac")
+
+        G = H * exp(-3 * s)
+
+        self.assertEqual(G.partfrac(), G,  "undef delay partfrac")
+        self.assertEqual(G.partfrac(True), G,  "undef delay partfrac")
+
+        F = G + exp(-4 * s)
+
+        self.assertEqual(F.partfrac(), F,  "undef delay sum partfrac")
+        self.assertEqual(F.partfrac(True), F,  "undef delay sum partfrac")
+
+        H = expr('F(s)') / (s**2 + 3 * s + 2)
+        
+        self.assertEqual(H.partfrac(), H,  "undef partfrac")
+        self.assertEqual(H.partfrac(True), H,  "undef partfrac")
+
+        G = H * exp(-3 * s)
+
+        self.assertEqual(G.partfrac(), G,  "undef delay partfrac")
+        self.assertEqual(G.partfrac(True), G,  "undef delay partfrac")
+
+        F = G + exp(-4 * s)
+
+        self.assertEqual(F.partfrac(), F,  "undef delay sum partfrac")
+        self.assertEqual(F.partfrac(True), F,  "undef delay sum partfrac")
+
+        H = expr('F(s)') / ((s + 2)**2)
+        
+        self.assertEqual(H.partfrac(), H,  "undef partfrac")
+        self.assertEqual(H.partfrac(True), H,  "undef partfrac")
+
+        G = H * exp(-3 * s)
+
+        self.assertEqual(G.partfrac(), G,  "undef delay partfrac")
+        self.assertEqual(G.partfrac(True), G,  "undef delay partfrac")
+
+        F = G + exp(-4 * s)
+
+        self.assertEqual(F.partfrac(), F,  "undef delay sum partfrac")
+        self.assertEqual(F.partfrac(True), F,  "undef delay sum partfrac")
+        
+
+    def test_mixedfrac(self):
+
+        H = expr('F(s)') / (s**2 + 3 * s + 6)
+        
+        self.assertEqual(H.mixedfrac(), H,  "undef mixedfrac")
+
+        G = H * exp(-3 * s)
+
+        self.assertEqual(G.mixedfrac(), G,  "undef delay mixedfrac")
+
+        F = G + exp(-4 * s)
+
+        self.assertEqual(F.mixedfrac(), F,  "undef delay sum mixedfrac")
+
+        H = expr('F(s)') / (s**2 + 3 * s + 2)
+        
+        self.assertEqual(H.mixedfrac(), H,  "undef mixedfrac")
+
+        G = H * exp(-3 * s)
+
+        self.assertEqual(G.mixedfrac(), G,  "undef delay mixedfrac")
+
+        F = G + exp(-4 * s)
+
+        self.assertEqual(F.mixedfrac(), F,  "undef delay sum mixedfrac")
+
+        H = expr('F(s)') / ((s + 2)**2)
+        
+        self.assertEqual(H.mixedfrac(), H,  "undef mixedfrac")
+
+        G = H * exp(-3 * s)
+
+        self.assertEqual(G.mixedfrac(), G,  "undef delay mixedfrac")
+
+        F = G + exp(-4 * s)
+
+        self.assertEqual(F.mixedfrac(), F,  "undef delay sum mixedfrac")
+        
